@@ -1,18 +1,18 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
 from flask.helpers import send_file
-from flask_restplus import inputs
-from flexget.api import api, APIResource
+from flask_restx import inputs
+from requests import RequestException
+
+from flexget.api import APIResource, api
 from flexget.api.app import APIError, BadRequest
 from flexget.utils.cache import cached_resource
-from requests import RequestException
 
 cached_api = api.namespace('cached', description='Cache remote resources')
 
 cached_parser = api.parser()
 cached_parser.add_argument('url', required=True, help='URL to cache')
-cached_parser.add_argument('force', type=inputs.boolean, default=False, help='Force fetching remote resource')
+cached_parser.add_argument(
+    'force', type=inputs.boolean, default=False, help='Force fetching remote resource'
+)
 
 
 @cached_api.route('/')
