@@ -1,48 +1,50 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
 import argparse
 import cgi
 import copy
 from datetime import datetime, timedelta
 from json import JSONEncoder
+from queue import Empty, Queue
 
-from flask import jsonify, Response, request
-from flask_restplus import inputs
-from queue import Queue, Empty
+from flask import Response, jsonify, request
+from flask_restx import inputs
 
-from flexget.api import api, APIResource
-from flexget.api.app import APIError, NotFoundError, Conflict, BadRequest, success_response, \
-    base_message_schema, etag
+from flexget.api import APIResource, api
+from flexget.api.app import (
+    APIError,
+    BadRequest,
+    Conflict,
+    NotFoundError,
+    base_message_schema,
+    etag,
+    success_response,
+)
 from flexget.config_schema import process_config
 from flexget.entry import Entry
 from flexget.event import event
+from flexget.log import capture_logs
 from flexget.options import get_parser
 from flexget.task import task_phases
-from flexget.utils import json
-from flexget.utils import requests
+from flexget.terminal import capture_console
+from flexget.utils import json, requests
 from flexget.utils.lazy_dict import LazyLookup
 
 # Tasks API
 tasks_api = api.namespace('tasks', description='Manage Tasks')
 
 
-class ObjectsContainer(object):
-    tasks_list_object = {'oneOf': [
-        {'type': 'array',
-         'items': {'$ref': '#/definitions/tasks.task'}},
-        {'type': 'array', 'items': {'type': 'string'}}
-    ]
+class ObjectsContainer:
+    tasks_list_object = {
+        'oneOf': [
+            {'type': 'array', 'items': {'$ref': '#/definitions/tasks.task'}},
+            {'type': 'array', 'items': {'type': 'string'}},
+        ]
     }
 
     task_input_object = {
         'type': 'object',
-        'properties': {
-            'name': {'type': 'string'},
-            'config': {'$ref': '/schema/plugins'}
-        },
+        'properties': {'name': {'type': 'string'}, 'config': {'$ref': '/schema/plugins'}},
         'required': ['name', 'config'],
-        'additionalProperties': False
+        'additionalProperties': False,
     }
 
     task_return_object = copy.deepcopy(task_input_object)
@@ -57,7 +59,8 @@ class ObjectsContainer(object):
                 'name': {'type': 'string'},
                 'current_phase': {'type': ['string', 'null']},
                 'current_plugin': {'type': ['string', 'null']},
-            }}
+            },
+        },
     }
 
     task_execution_results_schema = {
@@ -74,11 +77,14 @@ class ObjectsContainer(object):
                             'progress': {
                                 'type': 'object',
                                 'properties': {
-                                    'status': {'type': 'string', 'enum': ['pending', 'running', 'complete']},
+                                    'status': {
+                                        'type': 'string',
+                                        'enum': ['pending', 'running', 'complete'],
+                                    },
                                     'phase': {'type': 'string', 'enum': task_phases},
                                     'plugin': {'type': 'string'},
-                                    'percent': {'type': 'float'}
-                                }
+                                    'percent': {'type': 'float'},
+                                },
                             },
                             'summary': {
                                 'type': 'object',
@@ -89,64 +95,74 @@ class ObjectsContainer(object):
                                     'undecided': {'type': 'integer'},
                                     'aborted': {'type': 'boolean'},
                                     'abort_reason': {'type': 'string'},
-                                }
+                                },
                             },
                             'entry_dump': {'type': 'array', 'items': {'type': 'object'}},
-                            'log': {'type': 'string'}
-                        }
-                    }
-                }
+                            'log': {'type': 'string'},
+                        },
+                    },
+                },
             }
-        }
+        },
     }
 
     inject_input = {
         'type': 'object',
         'properties': {
-            'title': {'type': 'string',
-                      'description': 'Title of the entry. If not supplied it will be attempted to retrieve it from '
-                                     'URL headers'},
-            'url': {'type': 'string',
-                    'format': 'url',
-                    'description': 'URL of the entry'},
-            'force': {'type': 'boolean',
-                      'description': 'Prevent any plugins from rejecting this entry'},
-            'accept': {'type': 'boolean',
-                       'description': 'Accept this entry immediately upon injection (disregard task filters)'},
-            'fields': {'type': 'object',
-                       'description': 'A array of objects that can contain any other value for the entry'}
+            'title': {
+                'type': 'string',
+                'description': 'Title of the entry. If not supplied it will be attempted to retrieve it from '
+                'URL headers',
+            },
+            'url': {'type': 'string', 'format': 'url', 'description': 'URL of the entry'},
+            'force': {
+                'type': 'boolean',
+                'description': 'Prevent any plugins from rejecting this entry',
+            },
+            'accept': {
+                'type': 'boolean',
+                'description': 'Accept this entry immediately upon injection (disregard task filters)',
+            },
+            'fields': {
+                'type': 'object',
+                'description': 'A array of objects that can contain any other value for the entry',
+            },
         },
-        'required': ['url']
+        'required': ['url'],
     }
 
     task_execution_input = {
         'type': 'object',
         'properties': {
-            'tasks': {'type': 'array',
-                      'items': {'type': 'string'},
-                      'minItems': 1,
-                      'uniqueItems': True},
+            'tasks': {
+                'type': 'array',
+                'items': {'type': 'string'},
+                'minItems': 1,
+                'uniqueItems': True,
+            },
             'progress': {
                 'type': 'boolean',
                 'default': True,
-                'description': 'Include task progress updates'},
-            'summary': {
-                'type': 'boolean',
-                'default': True,
-                'description': 'Include task summary'},
+                'description': 'Include task progress updates',
+            },
+            'summary': {'type': 'boolean', 'default': True, 'description': 'Include task summary'},
             'entry_dump': {
                 'type': 'boolean',
                 'default': True,
-                'description': 'Include dump of entries including fields'},
-            'inject': {'type': 'array',
-                       'items': inject_input,
-                       'description': 'A List of entry objects'},
-            'loglevel': {'type': 'string',
-                         'description': 'Specify log level',
-                         'enum': ['critical', 'error', 'warning', 'info', 'verbose', 'debug', 'trace']}
+                'description': 'Include dump of entries including fields',
+            },
+            'inject': {
+                'type': 'array',
+                'items': inject_input,
+                'description': 'A List of entry objects',
+            },
+            'loglevel': {
+                'type': 'string',
+                'description': 'Specify log level',
+                'enum': ['critical', 'error', 'warning', 'info', 'verbose', 'debug', 'trace'],
+            },
         },
-        'required': ['tasks']
-
+        'required': ['tasks'],
     }
 
     params_return_schema = {'type': 'array', 'items': {'type': 'object'}}
@@ -156,14 +172,24 @@ tasks_list_schema = api.schema_model('tasks.list', ObjectsContainer.tasks_list_o
 task_input_schema = api.schema_model('tasks.task', ObjectsContainer.task_input_object)
 task_return_schema = api.schema_model('tasks.task', ObjectsContainer.task_return_object)
 task_api_queue_schema = api.schema_model('task.queue', ObjectsContainer.task_queue_schema)
-task_api_execute_schema = api.schema_model('task.execution', ObjectsContainer.task_execution_results_schema)
-task_execution_schema = api.schema_model('task_execution_input', ObjectsContainer.task_execution_input)
-task_execution_params = api.schema_model('tasks.execution_params', ObjectsContainer.params_return_schema)
+task_api_execute_schema = api.schema_model(
+    'task.execution', ObjectsContainer.task_execution_results_schema
+)
+task_execution_schema = api.schema_model(
+    'task_execution_input', ObjectsContainer.task_execution_input
+)
+task_execution_params = api.schema_model(
+    'tasks.execution_params', ObjectsContainer.params_return_schema
+)
 
-task_api_desc = 'Task config schema too large to display, you can view the schema using the schema API'
+task_api_desc = (
+    'Task config schema too large to display, you can view the schema using the schema API'
+)
 
 tasks_parser = api.parser()
-tasks_parser.add_argument('include_config', type=inputs.boolean, default=True, help='Include task config')
+tasks_parser.add_argument(
+    'include_config', type=inputs.boolean, default=True, help='Include task config'
+)
 
 
 @tasks_api.route('/')
@@ -175,13 +201,17 @@ class TasksAPI(APIResource):
     def get(self, session=None):
         """ List all tasks """
 
+        active_tasks = {
+            task: task_data
+            for task, task_data in self.manager.user_config.get('tasks', {}).items()
+            if not task.startswith('_')
+        }
+
         args = tasks_parser.parse_args()
         if not args.get('include_config'):
-            return jsonify(list(self.manager.user_config.get('tasks', {})))
+            return jsonify(list(active_tasks))
 
-        tasks = []
-        for name, config in self.manager.user_config.get('tasks', {}).items():
-            tasks.append({'name': name, 'config': config})
+        tasks = [{'name': name, 'config': config} for name, config in active_tasks.items()]
         return jsonify(tasks)
 
     @api.validate(task_input_schema, description='New task object')
@@ -203,7 +233,9 @@ class TasksAPI(APIResource):
             self.manager.config['tasks'] = {}
 
         task_schema_processed = copy.deepcopy(data)
-        errors = process_config(task_schema_processed, schema=task_input_schema.__schema__, set_defaults=True)
+        errors = process_config(
+            task_schema_processed, schema=task_input_schema.__schema__, set_defaults=True
+        )
 
         if errors:
             raise APIError('problem loading config, raise a BUG as this should not happen!')
@@ -260,7 +292,9 @@ class TaskAPI(APIResource):
 
         # Process the task config
         task_schema_processed = copy.deepcopy(data)
-        errors = process_config(task_schema_processed, schema=task_return_schema.__schema__, set_defaults=True)
+        errors = process_config(
+            task_schema_processed, schema=task_return_schema.__schema__, set_defaults=True
+        )
 
         if errors:
             raise APIError('problem loading config, raise a BUG as this should not happen!')
@@ -271,7 +305,9 @@ class TaskAPI(APIResource):
         self.manager.save_config()
         self.manager.config_changed()
 
-        rsp = jsonify({'name': new_task_name, 'config': self.manager.user_config['tasks'][new_task_name]})
+        rsp = jsonify(
+            {'name': new_task_name, 'config': self.manager.user_config['tasks'][new_task_name]}
+        )
         rsp.status_code = 200
         return rsp
 
@@ -293,15 +329,33 @@ class TaskAPI(APIResource):
 default_start_date = (datetime.now() - timedelta(weeks=1)).strftime('%Y-%m-%d')
 
 status_parser = api.parser()
-status_parser.add_argument('succeeded', type=inputs.boolean, default=True, help='Filter by success status')
-status_parser.add_argument('produced', type=inputs.boolean, default=True, store_missing=False,
-                           help='Filter by tasks that produced entries')
-status_parser.add_argument('start_date', type=inputs.datetime_from_iso8601, default=default_start_date,
-                           help='Filter by minimal start date. Example: \'2012-01-01\'')
-status_parser.add_argument('end_date', type=inputs.datetime_from_iso8601,
-                           help='Filter by maximal end date. Example: \'2012-01-01\'')
-status_parser.add_argument('limit', default=100, type=int,
-                           help='Limit return of executions per task, as that number can be huge')
+status_parser.add_argument(
+    'succeeded', type=inputs.boolean, default=True, help='Filter by success status'
+)
+status_parser.add_argument(
+    'produced',
+    type=inputs.boolean,
+    default=True,
+    store_missing=False,
+    help='Filter by tasks that produced entries',
+)
+status_parser.add_argument(
+    'start_date',
+    type=inputs.datetime_from_iso8601,
+    default=default_start_date,
+    help='Filter by minimal start date. Example: \'2012-01-01\'',
+)
+status_parser.add_argument(
+    'end_date',
+    type=inputs.datetime_from_iso8601,
+    help='Filter by maximal end date. Example: \'2012-01-01\'',
+)
+status_parser.add_argument(
+    'limit',
+    default=100,
+    type=int,
+    help='Limit return of executions per task, as that number can be huge',
+)
 
 
 def _task_info_dict(task):
@@ -343,7 +397,7 @@ inject_api = api.namespace('inject', description='Entry injection API')
 @tasks_api.route('/execute/params/')
 @api.doc(description='Available payload parameters for task execute')
 class TaskExecutionParams(APIResource):
-    @etag
+    @etag(cache_age=3600)
     @api.response(200, model=task_execution_params)
     def get(self, session=None):
         """ Execute payload parameters """
@@ -362,19 +416,33 @@ class TaskExecutionAPI(APIResource):
         """ Execute task and stream results """
         data = request.json
         for task in data.get('tasks'):
-            if task.lower() not in [t.lower() for t in self.manager.user_config.get('tasks', {}).keys()]:
+            if task.lower() not in [
+                t.lower() for t in self.manager.user_config.get('tasks', {}).keys()
+            ]:
                 raise NotFoundError('task %s does not exist' % task)
 
         queue = ExecuteLog()
         output = queue if data.get('loglevel') else None
-        stream = True if any(
-            arg[0] in ['progress', 'summary', 'loglevel', 'entry_dump'] for arg in data.items() if arg[1]) else False
+        stream = (
+            True
+            if any(
+                arg[0] in ['progress', 'summary', 'loglevel', 'entry_dump']
+                for arg in data.items()
+                if arg[1]
+            )
+            else False
+        )
         loglevel = data.pop('loglevel', None)
 
+        if loglevel:
+            loglevel = loglevel.upper()
+
         # This emulates the CLI command of using `--now` and `no-cache`
-        options = {'interval_ignore': data.pop('now', None),
-                   'nocache': data.pop('no_cache', None),
-                   'allow_manual': True}
+        options = {
+            'interval_ignore': data.pop('now', None),
+            'nocache': data.pop('no_cache', None),
+            'allow_manual': True,
+        }
 
         for option, value in data.items():
             options[option] = value
@@ -386,10 +454,14 @@ class TaskExecutionAPI(APIResource):
                 entry['url'] = item['url']
                 if not item.get('title'):
                     try:
-                        value, params = cgi.parse_header(requests.head(item['url']).headers['Content-Disposition'])
+                        value, params = cgi.parse_header(
+                            requests.head(item['url']).headers['Content-Disposition']
+                        )
                         entry['title'] = params['filename']
                     except KeyError:
-                        raise BadRequest('No title given, and couldn\'t get one from the URL\'s HTTP response')
+                        raise BadRequest(
+                            'No title given, and couldn\'t get one from the URL\'s HTTP response'
+                        )
 
                 else:
                     entry['title'] = item.get('title')
@@ -403,25 +475,29 @@ class TaskExecutionAPI(APIResource):
                 entries.append(entry)
             options['inject'] = entries
 
-        executed_tasks = self.manager.execute(options=options, output=output, loglevel=loglevel)
+        if output:
+            with capture_console(output), capture_logs(output, level=loglevel):
+                executed_tasks = self.manager.execute(options=options)
+        else:
+            executed_tasks = self.manager.execute(options=options)
 
         tasks_queued = []
 
         for task_id, task_name, task_event in executed_tasks:
             tasks_queued.append({'id': task_id, 'name': task_name, 'event': task_event})
-            _streams[task_id] = {
-                'queue': queue,
-                'last_update': datetime.now(),
-                'args': data
-            }
+            _streams[task_id] = {'queue': queue, 'last_update': datetime.now(), 'args': data}
 
         if not stream:
-            return jsonify({'tasks': [{'id': task['id'], 'name': task['name']} for task in tasks_queued]})
+            return jsonify(
+                {'tasks': [{'id': task['id'], 'name': task['name']} for task in tasks_queued]}
+            )
 
         def stream_response():
             # First return the tasks to execute
             yield '{"stream": ['
-            yield json.dumps({'tasks': [{'id': task['id'], 'name': task['name']} for task in tasks_queued]}) + ',\n'
+            yield json.dumps(
+                {'tasks': [{'id': task['id'], 'name': task['name']} for task in tasks_queued]}
+            ) + ',\n'
 
             while True:
                 try:
@@ -503,10 +579,10 @@ def update_stream(task, status='pending'):
         'status': status,
         'phase': task.current_phase,
         'plugin': task.current_plugin,
-        'percent': task.stream.get('percent', 0)
+        'percent': task.stream.get('percent', 0),
     }
 
-    task.stream['queue'].put(json.dumps({'progress': progress}))
+    task.stream['queue'].put(json.dumps({'progress': progress, 'task_id': task.id}))
 
 
 @event('task.execute.started')
@@ -525,19 +601,26 @@ def finish_task(task):
 
         if task.stream['args'].get('entry_dump'):
             entries = [entry.store for entry in task.entries]
-            task.stream['queue'].put(EntryDecoder().encode({'entry_dump': entries}))
+            task.stream['queue'].put(
+                EntryDecoder().encode({'entry_dump': entries, 'task_id': task.id})
+            )
 
         if task.stream['args'].get('summary'):
-            task.stream['queue'].put(json.dumps({
-                'summary': {
-                    'accepted': len(task.accepted),
-                    'rejected': len(task.rejected),
-                    'failed': len(task.failed),
-                    'undecided': len(task.undecided),
-                    'aborted': task.aborted,
-                    'abort_reason': task.abort_reason,
-                }
-            }))
+            task.stream['queue'].put(
+                json.dumps(
+                    {
+                        'summary': {
+                            'accepted': len(task.accepted),
+                            'rejected': len(task.rejected),
+                            'failed': len(task.failed),
+                            'undecided': len(task.undecided),
+                            'aborted': task.aborted,
+                            'abort_reason': task.abort_reason,
+                        },
+                        'task_id': task.id,
+                    }
+                )
+            )
 
 
 @event('task.execute.before_plugin')
