@@ -1,29 +1,29 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-from flexget.utils.template import get_template
-
-from future.moves.urllib.parse import urlparse, parse_qsl
-
+import datetime
 import os
 import re
-import logging
 from collections import defaultdict
-from datetime import datetime
+from typing import Any, Callable, Dict, Optional, Union
+from urllib.parse import parse_qsl, urlparse
 
 import jsonschema
-from jsonschema.compat import str_types, int_types
+from jsonschema.compat import int_types, str_types
+from loguru import logger
 
 from flexget.event import fire_event
 from flexget.utils import qualities, template
-from flexget.utils.tools import parse_timedelta, parse_episode_identifier
+from flexget.utils.template import get_template
+from flexget.utils.tools import parse_episode_identifier, parse_timedelta
+
+logger = logger.bind(name='config_schema')
 
 schema_paths = {}
-
-log = logging.getLogger('config_schema')
+CURRENT_SCHEMA_VERSION = 'http://json-schema.org/draft-04/schema#'
+# Type hint for json schemas. (If we upgrade to a newer json schema version, the type might allow more than dicts.)
+JsonSchema = Dict[str, Any]
 
 
 # TODO: Rethink how config key and schema registration work
-def register_schema(path, schema):
+def register_schema(path: str, schema: Union[JsonSchema, Callable[..., JsonSchema]]):
     """
     Register `schema` to be available at `path` for $refs
 
@@ -34,10 +34,10 @@ def register_schema(path, schema):
 
 
 # Validator that handles root structure of config.
-_root_config_schema = None
+_root_config_schema: Optional[JsonSchema] = None
 
 
-def register_config_key(key, schema, required=False):
+def register_config_key(key: str, schema: JsonSchema, required: bool = False):
     """ Registers a valid root level key for the config.
 
     :param string key:
@@ -47,23 +47,29 @@ def register_config_key(key, schema, required=False):
     :param bool required:
       Specify whether this is a mandatory key.
     """
-    _root_config_schema['properties'][key] = schema
+    root_schema = get_schema()
+    root_schema['properties'][key] = schema
     if required:
-        _root_config_schema.setdefault('required', []).append(key)
+        root_schema.setdefault('required', []).append(key)
     register_schema('/schema/config/%s' % key, schema)
 
 
-def get_schema():
+def get_schema() -> JsonSchema:
     global _root_config_schema
     if _root_config_schema is None:
-        _root_config_schema = {'type': 'object', 'properties': {}, 'additionalProperties': False}
+        _root_config_schema = {
+            'type': 'object',
+            'properties': {},
+            'additionalProperties': False,
+            '$schema': CURRENT_SCHEMA_VERSION,
+        }
         fire_event('config.register')
-        # TODO: Is /schema/root this the best place for this?
+        # TODO: Is /schema/config this the best place for this?
         register_schema('/schema/config', _root_config_schema)
     return _root_config_schema
 
 
-def one_or_more(schema, unique_items=False):
+def one_or_more(schema: JsonSchema, unique_items: bool = False) -> JsonSchema:
     """
     Helper function to construct a schema that validates items matching `schema` or an array
     containing items matching `schema`.
@@ -73,13 +79,19 @@ def one_or_more(schema, unique_items=False):
     schema.setdefault('title', 'single value')
     return {
         'oneOf': [
-            {'title': 'multiple values', 'type': 'array', 'items': schema, 'minItems': 1, 'uniqueItems': unique_items},
-            schema
+            {
+                'title': 'multiple values',
+                'type': 'array',
+                'items': schema,
+                'minItems': 1,
+                'uniqueItems': unique_items,
+            },
+            schema,
         ]
     }
 
 
-def resolve_ref(uri):
+def resolve_ref(uri: str) -> JsonSchema:
     """
     Finds and returns a schema pointed to by `uri` that has been registered in the register_schema function.
     """
@@ -87,12 +99,13 @@ def resolve_ref(uri):
     if parsed.path in schema_paths:
         schema = schema_paths[parsed.path]
         if callable(schema):
-            return schema(**dict(parse_qsl(parsed.query)))
+            schema = schema(**dict(parse_qsl(parsed.query)))
+        schema = {'$schema': CURRENT_SCHEMA_VERSION, **schema}
         return schema
     raise jsonschema.RefResolutionError("%s could not be resolved" % uri)
 
 
-def process_config(config, schema=None, set_defaults=True):
+def process_config(config: Any, schema: Optional[JsonSchema] = None, set_defaults: bool = True):
     """
     Validates the config, and sets defaults within it if `set_defaults` is set.
     If schema is not given, uses the root config schema.
@@ -117,18 +130,18 @@ def process_config(config, schema=None, set_defaults=True):
     return errors
 
 
-def parse_time(time_string):
+def parse_time(time_string: str) -> datetime.time:
     """Parse a time string from the config into a :class:`datetime.time` object."""
     formats = ['%I:%M %p', '%H:%M', '%H:%M:%S']
     for f in formats:
         try:
-            return datetime.strptime(time_string, f).time()
+            return datetime.datetime.strptime(time_string, f).time()
         except ValueError:
             continue
     raise ValueError('invalid time `%s`' % time_string)
 
 
-def parse_interval(interval_string):
+def parse_interval(interval_string: str) -> datetime.timedelta:
     """Takes an interval string from the config and turns it into a :class:`datetime.timedelta` object."""
     regexp = r'^\d+ (second|minute|hour|day|week)s?$'
     if not re.match(regexp, interval_string):
@@ -136,8 +149,8 @@ def parse_interval(interval_string):
     return parse_timedelta(interval_string)
 
 
-def parse_percent(percent_input):
-    """Takes a size string from the config and turns it into int(bytes)."""
+def parse_percent(percent_input: str) -> float:
+    """Takes a percent string from the config and turns it into a float."""
     percent_input = percent_input.rstrip('%')
     try:
         return float(percent_input)
@@ -145,7 +158,7 @@ def parse_percent(percent_input):
         raise ValueError("should be in format '0-x%'")
 
 
-def parse_size(size_input):
+def parse_size(size_input: str) -> int:
     """Takes a size string from the config and turns it into int(bytes)."""
     prefixes = [None, 'K', 'M', 'G', 'T', 'P']
     try:
@@ -165,7 +178,7 @@ def parse_size(size_input):
 class RefResolver(jsonschema.RefResolver):
     def __init__(self, *args, **kwargs):
         kwargs.setdefault('handlers', {'': resolve_ref})
-        super(RefResolver, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
 
 format_checker = jsonschema.FormatChecker(('email',))
@@ -240,7 +253,7 @@ def is_path(instance):
     pat = re.compile(r'{[{%].*[}%]}')
     result = pat.search(instance)
     if result:
-        instance = os.path.dirname(instance[0:result.start()])
+        instance = os.path.dirname(instance[0 : result.start()])
     if os.path.isdir(os.path.expanduser(instance)):
         return True
     raise ValueError('`%s` does not exist' % instance)
@@ -251,8 +264,11 @@ def is_path(instance):
 def is_url(instance):
     if not isinstance(instance, str_types):
         return True
-    regexp = ('(' + '|'.join(['ftp', 'http', 'https', 'file', 'udp', 'socks5']) +
-              '):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?')
+    regexp = (
+        '('
+        + '|'.join(['ftp', 'http', 'https', 'file', 'udp', 'socks5h?'])
+        + r'):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?'
+    )
     return re.match(regexp, instance)
 
 
@@ -302,14 +318,20 @@ def set_error_message(error):
         if error.cause:
             error.message = str(error.cause)
     elif error.validator == 'enum':
-        error.message = 'Must be one of the following: %s' % ', '.join(map(str, error.validator_value))
+        error.message = 'Must be one of the following: %s' % ', '.join(
+            map(str, error.validator_value)
+        )
     elif error.validator == 'additionalProperties':
         if error.validator_value is False:
-            extras = set(jsonschema._utils.find_additional_properties(error.instance, error.schema))
+            extras = set(
+                jsonschema._utils.find_additional_properties(error.instance, error.schema)
+            )
             if len(extras) == 1:
                 error.message = 'The key `%s` is not valid here.' % extras.pop()
             else:
-                error.message = 'The keys %s are not valid here.' % ', '.join('`%s`' % e for e in extras)
+                error.message = 'The keys %s are not valid here.' % ', '.join(
+                    '`%s`' % e for e in extras
+                )
     else:
         # Remove u'' string representation from jsonschema error messages
         error.message = re.sub('u\'(.*?)\'', '`\\1`', error.message)
@@ -367,7 +389,9 @@ def validate_properties_w_defaults(validator, properties, instance, schema):
     for key, subschema in properties.items():
         if 'default' in subschema:
             instance.setdefault(key, subschema['default'])
-    for error in jsonschema.Draft4Validator.VALIDATORS["properties"](validator, properties, instance, schema):
+    for error in jsonschema.Draft4Validator.VALIDATORS["properties"](
+        validator, properties, instance, schema
+    ):
         yield error
 
 
@@ -385,13 +409,9 @@ def validate_oneOf(validator, oneOf, instance, schema):
 
 def validate_deprecated(validator, message, instance, schema):
     """Not really a validator, just warns if deprecated section of config is being used."""
-    log.warning(message)
+    logger.warning(message)
 
 
-validators = {
-    'anyOf': validate_anyOf,
-    'oneOf': validate_oneOf,
-    'deprecated': validate_deprecated
-}
+validators = {'anyOf': validate_anyOf, 'oneOf': validate_oneOf, 'deprecated': validate_deprecated}
 
 SchemaValidator = jsonschema.validators.extend(jsonschema.Draft4Validator, validators)
