@@ -1,16 +1,49 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-from future.utils import PY2, native_str, text_type
-
-import copy
 import functools
-import logging
+import types
+import warnings
+from enum import Enum
+from typing import Callable, Iterable, Mapping, Optional, Sequence, Union
 
-from flexget.plugin import PluginError
+from loguru import logger
+
+from flexget import plugin
 from flexget.utils.lazy_dict import LazyDict, LazyLookup
-from flexget.utils.template import render_from_entry, FlexGetTemplate
+from flexget.utils.serialization import Serializer, deserialize, serialize
+from flexget.utils.template import FlexGetTemplate, render_from_entry
 
-log = logging.getLogger('entry')
+logger = logger.bind(name='entry')
+
+
+class EntryState(Enum):
+    ACCEPTED = 'accepted'
+    REJECTED = 'rejected'
+    FAILED = 'failed'
+    UNDECIDED = 'undecided'
+
+    @property
+    def color(self) -> str:
+        return {
+            self.ACCEPTED: 'green',
+            self.REJECTED: 'red',
+            self.FAILED: 'RED',
+            self.UNDECIDED: 'dim',
+        }[self]
+
+    @property
+    def log_markup(self) -> str:
+        return f'<{self.color}>{self.value.upper()}</>'
+
+    # __str__, __eq__, and __hash__ are make these states almost interchangeable with the old plain string states
+    def __str__(self):
+        return self.value
+
+    def __eq__(self, other):
+        if isinstance(other, str):
+            return self.value == other
+        return super().__eq__(other)
+
+    def __hash__(self):
+        return hash(self.value)
 
 
 class EntryUnicodeError(Exception):
@@ -24,7 +57,7 @@ class EntryUnicodeError(Exception):
         return 'Entry strings must be unicode: %s (%r)' % (self.key, self.value)
 
 
-class Entry(LazyDict):
+class Entry(LazyDict, Serializer):
     """
     Represents one item in task. Must have `url` and *title* fields.
 
@@ -39,12 +72,12 @@ class Entry(LazyDict):
     """
 
     def __init__(self, *args, **kwargs):
-        super(Entry, self).__init__()
+        super().__init__()
         self.traces = []
-        self.snapshots = {}
-        self._state = 'undecided'
+        self._state = EntryState.UNDECIDED
         self._hooks = {'accept': [], 'reject': [], 'fail': [], 'complete': []}
         self.task = None
+        self.lazy_lookups = []
 
         if len(args) == 2:
             kwargs['title'] = args[0]
@@ -54,7 +87,7 @@ class Entry(LazyDict):
         # Make sure constructor does not escape our __setitem__ enforcement
         self.update(*args, **kwargs)
 
-    def trace(self, message, operation=None, plugin=None):
+    def trace(self, message: str, operation: Optional[str] = None, plugin: Optional[str] = None):
         """
         Adds trace message to the entry which should contain useful information about why
         plugin did not operate on entry. Accept and Reject messages are added to trace automatically.
@@ -69,7 +102,7 @@ class Entry(LazyDict):
         if item not in self.traces:
             self.traces.append(item)
 
-    def run_hooks(self, action, **kwargs):
+    def run_hooks(self, action: str, **kwargs):
         """
         Run hooks that have been registered for given ``action``.
 
@@ -79,7 +112,7 @@ class Entry(LazyDict):
         for func in self._hooks[action]:
             func(self, **kwargs)
 
-    def add_hook(self, action, func, **kwargs):
+    def add_hook(self, action: str, func: Callable, **kwargs):
         """
         Add a hook for ``action`` to this entry.
 
@@ -93,7 +126,7 @@ class Entry(LazyDict):
         except KeyError:
             raise ValueError('`%s` is not a valid entry action' % action)
 
-    def on_accept(self, func, **kwargs):
+    def on_accept(self, func: Callable, **kwargs):
         """
         Register a function to be called when this entry is accepted.
 
@@ -102,7 +135,7 @@ class Entry(LazyDict):
         """
         self.add_hook('accept', func, **kwargs)
 
-    def on_reject(self, func, **kwargs):
+    def on_reject(self, func: Callable, **kwargs):
         """
         Register a function to be called when this entry is rejected.
 
@@ -111,7 +144,7 @@ class Entry(LazyDict):
         """
         self.add_hook('reject', func, **kwargs)
 
-    def on_fail(self, func, **kwargs):
+    def on_fail(self, func: Callable, **kwargs):
         """
         Register a function to be called when this entry is failed.
 
@@ -120,7 +153,7 @@ class Entry(LazyDict):
         """
         self.add_hook('fail', func, **kwargs)
 
-    def on_complete(self, func, **kwargs):
+    def on_complete(self, func: Callable, **kwargs):
         """
         Register a function to be called when a :class:`Task` has finished processing this entry.
 
@@ -129,34 +162,34 @@ class Entry(LazyDict):
         """
         self.add_hook('complete', func, **kwargs)
 
-    def accept(self, reason=None, **kwargs):
+    def accept(self, reason: Optional[str] = None, **kwargs):
         if self.rejected:
-            log.debug('tried to accept rejected %r' % self)
+            logger.debug('tried to accept rejected {!r}', self)
         elif not self.accepted:
-            self._state = 'accepted'
+            self._state = EntryState.ACCEPTED
             self.trace(reason, operation='accept')
             # Run entry on_accept hooks
             self.run_hooks('accept', reason=reason, **kwargs)
 
-    def reject(self, reason=None, **kwargs):
+    def reject(self, reason: Optional[str] = None, **kwargs):
         # ignore rejections on immortal entries
         if self.get('immortal'):
             reason_str = '(%s)' % reason if reason else ''
-            log.info('Tried to reject immortal %s %s' % (self['title'], reason_str))
+            logger.info('Tried to reject immortal {} {}', self['title'], reason_str)
             self.trace('Tried to reject immortal %s' % reason_str)
             return
         if not self.rejected:
-            self._state = 'rejected'
+            self._state = EntryState.REJECTED
             self.trace(reason, operation='reject')
             # Run entry on_reject hooks
             self.run_hooks('reject', reason=reason, **kwargs)
 
-    def fail(self, reason=None, **kwargs):
-        log.debug('Marking entry \'%s\' as failed' % self['title'])
+    def fail(self, reason: Optional[str] = None, **kwargs):
+        logger.debug("Marking entry '{}' as failed", self['title'])
         if not self.failed:
-            self._state = 'failed'
+            self._state = EntryState.FAILED
             self.trace(reason, operation='fail')
-            log.error('Failed %s (%s)' % (self['title'], reason))
+            logger.error('Failed {} ({})', self['title'], reason)
             # Run entry on_fail hooks
             self.run_hooks('fail', reason=reason, **kwargs)
 
@@ -165,65 +198,61 @@ class Entry(LazyDict):
         self.run_hooks('complete', **kwargs)
 
     @property
-    def state(self):
+    def state(self) -> EntryState:
         return self._state
 
     @property
-    def accepted(self):
-        return self._state == 'accepted'
+    def accepted(self) -> bool:
+        return self._state == EntryState.ACCEPTED
 
     @property
-    def rejected(self):
-        return self._state == 'rejected'
+    def rejected(self) -> bool:
+        return self._state == EntryState.REJECTED
 
     @property
-    def failed(self):
-        return self._state == 'failed'
+    def failed(self) -> bool:
+        return self._state == EntryState.FAILED
 
     @property
-    def undecided(self):
-        return self._state == 'undecided'
+    def undecided(self) -> bool:
+        return self._state == EntryState.UNDECIDED
 
     def __setitem__(self, key, value):
         # Enforce unicode compatibility.
-        if PY2 and isinstance(value, native_str):
-            # Allow Python 2's implicit string decoding, but fail now instead of when entry fields are used.
-            # If encoding is anything but ascii, it should be decoded it to text before setting an entry field
-            try:
-                value = value.decode('ascii')
-            except UnicodeDecodeError:
-                raise EntryUnicodeError(key, value)
-        elif isinstance(value, bytes):
+        if isinstance(value, bytes):
             raise EntryUnicodeError(key, value)
         # Coerce any enriched strings (such as those returned by BeautifulSoup) to plain strings to avoid serialization
         # troubles.
-        elif isinstance(value, text_type) and type(value) != text_type:  # pylint: disable=unidiomatic-typecheck
-            value = text_type(value)
+        elif (
+            isinstance(value, str) and type(value) != str
+        ):  # pylint: disable=unidiomatic-typecheck
+            value = str(value)
 
         # url and original_url handling
         if key == 'url':
             if not isinstance(value, (str, LazyLookup)):
-                raise PluginError('Tried to set %r url to %r' % (self.get('title'), value))
+                raise plugin.PluginError('Tried to set %r url to %r' % (self.get('title'), value))
             self.setdefault('original_url', value)
 
         # title handling
         if key == 'title':
             if not isinstance(value, (str, LazyLookup)):
-                raise PluginError('Tried to set title to %r' % value)
+                raise plugin.PluginError('Tried to set title to %r' % value)
+            self.setdefault('original_title', value)
 
         try:
-            log.trace('ENTRY SET: %s = %r' % (key, value))
+            logger.trace('ENTRY SET: {} = {!r}', key, value)
         except Exception as e:
-            log.debug('trying to debug key `%s` value threw exception: %s' % (key, e))
+            logger.debug('trying to debug key `{}` value threw exception: {}', key, e)
 
-        super(Entry, self).__setitem__(key, value)
+        super().__setitem__(key, value)
 
-    def safe_str(self):
-        return '%s | %s' % (self['title'], self['url'])
+    def safe_str(self) -> str:
+        return f'{self["title"]} | {self["url"]}'
 
     # TODO: this is too manual, maybe we should somehow check this internally and throw some exception if
     # application is trying to operate on invalid entry
-    def isvalid(self):
+    def isvalid(self) -> bool:
         """
         :return: True if entry is valid. Return False if this cannot be used.
         :rtype: bool
@@ -238,23 +267,9 @@ class Entry(LazyDict):
             return False
         return True
 
-    def take_snapshot(self, name):
-        """
-        Takes a snapshot of the entry under *name*. Snapshots can be accessed via :attr:`.snapshots`.
-        :param string name: Snapshot name
-        """
-        snapshot = {}
-        for field, value in self.items():
-            try:
-                snapshot[field] = copy.deepcopy(value)
-            except TypeError:
-                log.warning('Unable to take `%s` snapshot for field `%s` in `%s`' % (name, field, self['title']))
-        if snapshot:
-            if name in self.snapshots:
-                log.warning('Snapshot `%s` is being overwritten for `%s`' % (name, self['title']))
-            self.snapshots[name] = snapshot
-
-    def update_using_map(self, field_map, source_item, ignore_none=False):
+    def update_using_map(
+        self, field_map: dict, source_item: Union[dict, object], ignore_none: bool = False
+    ):
         """
         Populates entry fields from a source object using a dictionary that maps from entry field names to
         attributes (or keys) in the source object.
@@ -278,26 +293,136 @@ class Entry(LazyDict):
                 continue
             self[field] = v
 
-    def render(self, template):
+    def render(self, template: Union[str, FlexGetTemplate], native: bool = False) -> str:
         """
         Renders a template string based on fields in the entry.
 
         :param template: A template string or FlexGetTemplate that uses jinja2 or python string replacement format.
+        :param native: If True, and the rendering result can be all native python types, not just strings.
         :return: The result of the rendering.
         :rtype: string
         :raises RenderError: If there is a problem.
         """
         if not isinstance(template, (str, FlexGetTemplate)):
             raise ValueError(
-                'Trying to render non string template or unrecognized template format, got %s' % repr(template))
-        log.trace('rendering: %s', template)
-        return render_from_entry(template, self)
+                'Trying to render non string template or unrecognized template format, got %s'
+                % repr(template)
+            )
+        logger.trace('rendering: {}', template)
+        return render_from_entry(template, self, native=native)
+
+    @classmethod
+    def serialize(cls, entry: 'Entry') -> dict:
+        fields = {}
+        for key in entry:
+            if key.startswith('_') or entry.is_lazy(key):
+                continue
+            try:
+                fields[key] = serialize(entry[key])
+            except TypeError as exc:
+                logger.debug('field {} was not serializable. {}', key, exc)
+        lazy_lookups = []
+        for ll in entry.lazy_lookups:
+            try:
+                lazy_lookups.append(serialize(ll))
+            except TypeError as exc:
+                logger.exception(
+                    'BUG: Lazy lookup was not compatible with serialization. Please file a bug report: {}',
+                    exc,
+                )
+        return {'fields': fields, 'lazy_lookups': lazy_lookups}
+
+    @classmethod
+    def deserialize(cls, data, version) -> 'Entry':
+        result = cls()
+        for key, value in data['fields'].items():
+            result[key] = deserialize(value)
+        for lazy_lookup in deserialize(data['lazy_lookups']):
+            result.add_lazy_fields(*lazy_lookup)
+        return result
+
+    def add_lazy_fields(
+        self,
+        lazy_func: Union[Callable[['Entry'], None], str],
+        fields: Iterable[str],
+        args: Sequence = None,
+        kwargs: Mapping = None,
+    ):
+        """
+        Add lazy fields to an entry.
+        :param lazy_func: should be a funciton previously registered with the `register_lazy_func` decorator,
+            or the name it was registered under.
+        :param fields: list of fields this function will fill
+        :param args: Arguments that will be passed to the lazy lookup function when called.
+        :param kwargs: Keyword arguments which will be passed to the lazy lookup function when called.
+        """
+        if not isinstance(lazy_func, str):
+            lazy_func = getattr(lazy_func, 'lazy_func_id', None)
+        if lazy_func not in lazy_func_registry:
+            raise ValueError(
+                'Lazy lookup functions/methods must be registered with the `register_lazy_lookup` decorator'
+            )
+        # Make sure fields is a plain list. If it is a dict, (or some other iterable) we may try to serialize
+        # extra/unserializable stuff.
+        fields = list(fields)
+        func = lazy_func_registry[lazy_func]
+        super().register_lazy_func(func.function, fields, args, kwargs)
+        self.lazy_lookups.append((lazy_func, fields, args, kwargs))
+
+    def register_lazy_func(self, func, keys):
+        """
+        DEPRECATED. Use `add_lazy_fields` instead.
+        """
+        warnings.warn(
+            '`register_lazy_func` is deprecated. `add_lazy_fields` should be used instead. '
+            'This plugin should be updated to work with the latest versions of FlexGet',
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        super().register_lazy_func(func, keys, [], {})
 
     def __eq__(self, other):
-        return self.get('title') == other.get('title') and self.get('original_url') == other.get('original_url')
+        return self.get('original_title') == other.get('original_title') and self.get(
+            'original_url'
+        ) == other.get('original_url')
 
     def __hash__(self):
-        return hash(self.get('title', '') + self.get('original_url', ''))
+        return hash(self.get('original_title', '') + self.get('original_url', ''))
 
     def __repr__(self):
         return '<Entry(title=%s,state=%s)>' % (self['title'], self._state)
+
+
+lazy_func_registry = {}
+
+
+class LazyFunc:
+    def __init__(self, lazy_func_name: str):
+        self.name = lazy_func_name
+        self._func = None
+
+    def __call__(self, func: Callable) -> Callable:
+        if self.name in lazy_func_registry:
+            raise Exception(
+                f'The name {self.name} is already registered to another lazy function.'
+            )
+        func.lazy_func_id = self.name
+        self._func = func
+        lazy_func_registry[self.name] = self
+        return func
+
+    @property
+    def function(self) -> Callable:
+        if '.' in self._func.__qualname__:
+            # This is a method of a plugin class, bind the function to the plugin instance
+            plugin_class_name = self._func.__qualname__.split('.')[0]
+            for p in plugin.plugins.values():
+                if p.plugin_class.__name__ == plugin_class_name:
+                    return types.MethodType(self._func, p.instance)
+            raise Exception(
+                f'lazy lookups must be functions, or methods of a registered plugin class. {self._func!r} is not'
+            )
+        return self._func
+
+
+register_lazy_lookup = LazyFunc
