@@ -1,14 +1,11 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
-import logging
+from loguru import logger
 
 from flexget import options
 from flexget.event import event
-from flexget.terminal import console
 from flexget.manager import Session
+from flexget.terminal import console
 
-log = logging.getLogger('perftests')
+logger = logger.bind(name='perftests')
 
 TESTS = ['imdb_query']
 
@@ -27,7 +24,9 @@ def cli_perf_test(manager, options):
 
 def imdb_query(session):
     import time
-    from flexget.plugins.metainfo.imdb_lookup import Movie
+
+    # NOTE: importing other plugins directly is discouraged
+    from flexget.components.imdb.db import Movie
     from flexget.plugins.cli.performance import log_query_count
     from sqlalchemy.sql.expression import select
     from progressbar import ProgressBar, Percentage, Bar, ETA
@@ -35,13 +34,13 @@ def imdb_query(session):
 
     imdb_urls = []
 
-    log.info('Getting imdb_urls ...')
+    logger.info('Getting imdb_urls ...')
     # query so that we avoid loading whole object (maybe cached?)
     for _, url in session.execute(select([Movie.id, Movie.url])):
         imdb_urls.append(url)
-    log.info('Got %i urls from database' % len(imdb_urls))
+    logger.info('Got {} urls from database', len(imdb_urls))
     if not imdb_urls:
-        log.info('so .. aborting')
+        logger.info('so .. aborting')
         return
 
     # commence testing
@@ -57,10 +56,12 @@ def imdb_query(session):
         # movie = session.query(Movie).filter(Movie.url == url).first()
         # movie = session.query(Movie).options(subqueryload(Movie.genres)).filter(Movie.url == url).one()
 
-        movie = session.query(Movie). \
-            options(joinedload_all(Movie.genres, Movie.languages,
-                                   Movie.actors, Movie.directors)). \
-            filter(Movie.url == url).first()
+        movie = (
+            session.query(Movie)
+            .options(joinedload_all(Movie.genres, Movie.languages, Movie.actors, Movie.directors))
+            .filter(Movie.url == url)
+            .first()
+        )
 
         # access it's members so they're loaded
         [x.name for x in movie.genres]
@@ -70,7 +71,7 @@ def imdb_query(session):
 
     log_query_count('test')
     took = time.time() - start_time
-    log.debug('Took %.2f seconds to query %i movies' % (took, len(imdb_urls)))
+    logger.debug('Took %.2f seconds to query %i movies' % (took, len(imdb_urls)))
 
 
 @event('options.register')
