@@ -1,29 +1,26 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-from future.moves.xmlrpc import client as xmlrpc_client
-from future.moves.urllib.parse import urlparse, urljoin
-from future.utils import native_str
-
-import logging
 import os
-import socket
 import re
+import socket
+from io import BytesIO
 from time import sleep
+from urllib.parse import urljoin, urlparse, urlsplit
+from xmlrpc import client as xmlrpc_client
+
+from loguru import logger
+from requests.auth import HTTPBasicAuth, HTTPDigestAuth
 
-from flexget.utils.template import RenderError
-from flexget.utils.pathscrub import pathscrub
 from flexget import plugin
-from flexget.event import event
-from flexget.entry import Entry
 from flexget.config_schema import one_or_more
+from flexget.entry import Entry
+from flexget.event import event
 from flexget.utils.bittorrent import Torrent, is_torrent_file
+from flexget.utils.pathscrub import pathscrub
+from flexget.utils.template import RenderError
 
-from requests.auth import HTTPDigestAuth, HTTPBasicAuth
-
-log = logging.getLogger('rtorrent')
+logger = logger.bind(name='rtorrent')
 
 
-class _Method(object):
+class _Method:
     # some magic to bind an XML-RPC method to an RPC server.
     # supports "nested" methods (e.g. examples.getStateName)
 
@@ -63,9 +60,11 @@ class HTTPDigestTransport(xmlrpc_client.Transport):
 
         # if status code is 401, it means we used the wrong auth method
         if response.status_code == 401:
-            log.warning('%s auth failed. Retrying with %s. Please change your config.',
-                        'Digest' if self.__digest_auth else 'Basic',
-                        'Basic' if self.__digest_auth else 'Digest')
+            logger.warning(
+                '{} auth failed. Retrying with {}. Please change your config.',
+                'Digest' if self.__digest_auth else 'Basic',
+                'Basic' if self.__digest_auth else 'Digest',
+            )
             self.__digest_auth = not self.__digest_auth
 
             auth = self.get_auth()
@@ -79,7 +78,7 @@ class HTTPDigestTransport(xmlrpc_client.Transport):
         if self.__digest_auth:
             return HTTPDigestAuth(self.__username, self.__password)
         return HTTPBasicAuth(self.__username, self.__password)
-    
+
     def send_request(self, url, auth, data):
         return self.__session.post(url, auth=auth, data=data, raise_status=False)
 
@@ -87,7 +86,7 @@ class HTTPDigestTransport(xmlrpc_client.Transport):
         p, u = self.getparser()
 
         if self.verbose:
-            log.info('body: %s', repr(response))
+            logger.info('body: {!r}', response)
 
         p.feed(response.content)
         p.close()
@@ -95,154 +94,99 @@ class HTTPDigestTransport(xmlrpc_client.Transport):
         return u.close()
 
 
+def encode_netstring(input):
+    return str(len(input)).encode() + b':' + input + b','
+
+
+def encode_header(key, value):
+    return key + b'\x00' + value + b'\x00'
+
+
 class SCGITransport(xmlrpc_client.Transport):
-    """ Used to override the default xmlrpclib transport to support SCGI """
+    """
+    Public domain SCGITrannsport implementation from:
+    https://github.com/JohnDoee/autotorrent/blob/develop/autotorrent/scgitransport.py
+    """
 
     def __init__(self, *args, **kwargs):
-        self.verbose = 0
+        self.socket_path = kwargs.pop('socket_path', '')
         xmlrpc_client.Transport.__init__(self, *args, **kwargs)
 
-    def request(self, host, handler, request_body, verbose=False):
-        return self.single_request(host, handler, request_body, verbose)
+    def single_request(self, host, handler, request_body, verbose=False):
+        self.verbose = verbose
+        if self.socket_path:
+            s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+            s.connect(self.socket_path)
+        else:
+            host, port = host.split(':')
+            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            s.connect((host, int(port)))
 
-    def single_request(self, host, handler, request_body, verbose=0):
-        # Add SCGI headers to the request.
-        headers = [('CONTENT_LENGTH', native_str(len(request_body))), ('SCGI', '1')]
-        header = '\x00'.join(['%s\x00%s' % (key, value) for key, value in headers]) + '\x00'
-        header = '%d:%s' % (len(header), header)
-        request_body = '%s,%s' % (header, request_body)
+        request = encode_header(b'CONTENT_LENGTH', str(len(request_body)).encode())
+        request += encode_header(b'SCGI', b'1')
+        request += encode_header(b'REQUEST_METHOD', b'POST')
+        request += encode_header(b'REQUEST_URI', handler.encode())
 
-        sock = None
+        request = encode_netstring(request)
+        request += request_body
 
-        try:
-            if host:
-                parsed_host = urlparse(host)
-                host = parsed_host.hostname
-                port = parsed_host.port
-
-                addr_info = socket.getaddrinfo(host, port, socket.AF_INET, socket.SOCK_STREAM)
-                sock = socket.socket(*addr_info[0][:3])
-                sock.connect(addr_info[0][4])
-            else:
-                sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
-                sock.connect(handler)
-
-            self.verbose = verbose
-
-            sock.sendall(request_body.encode())
-
-            return self.parse_response(sock.makefile())
-        finally:
-            if sock:
-                sock.close()
-
-    def parse_response(self, response):
-        p, u = self.getparser()
-
-        response_body = ''
+        s.send(request)
 
+        response = b''
         while True:
-            data = response.read(1024)
-            if not data:
+            r = s.recv(1024)
+            if not r:
                 break
-            response_body += data
+            response += r
 
-        if self.verbose:
-            log.info('body: %s', repr(response_body))
+        response_body = BytesIO(b'\r\n\r\n'.join(response.split(b'\r\n\r\n')[1:]))
 
-        # Remove SCGI headers from the response.
-        _, response_body = re.split(r'\n\s*?\n', response_body, maxsplit=1)
-        p.feed(response_body)
-        p.close()
-
-        return u.close()
+        return self.parse_response(response_body)
 
 
-class SCGIServerProxy(object):
-    """ Enable connection to SCGI proxy """
-
-    def __init__(self, uri, transport=None, encoding=None,
-                 verbose=False, allow_none=False, use_datetime=False):
-        parsed_url = urlparse(uri)
-        self.__host = uri if parsed_url.scheme else None
-        self.__handler = parsed_url.path
-        if not self.__handler:
-            self.__handler = '/'
-
-        if not transport:
-            transport = SCGITransport(use_datetime=use_datetime)
-        self.__transport = transport
-        self.__encoding = encoding or 'utf-8'
-        self.__verbose = verbose
-        self.__allow_none = allow_none
-
-    def __close(self):
-        self.__transport.close()
-
-    def __request(self, method_name, params):
-        # call a method on the remote server
-
-        request = xmlrpc_client.dumps(params, method_name, encoding=self.__encoding,
-                                      allow_none=self.__allow_none).encode(self.__encoding)
-
-        response = self.__transport.request(
-            self.__host,
-            self.__handler,
-            request.decode('utf-8'),
-            verbose=self.__verbose
-        )
-
-        if len(response) == 1:
-            response = response[0]
-
-        return response
-
-    def __repr__(self):
-        return (
-            "<ServerProxy for %s%s>" %
-            (self.__host, self.__handler)
-        )
-
-    __str__ = __repr__
-
-    def __getattr__(self, name):
-        # magic method dispatcher
-        return _Method(self.__request, name)
-
-    # note: to call a remote object with an non-standard name, use
-    # result getattr(server, "strange-python-name")(args)
-
-    def __call__(self, attr):
-        """A workaround to get special attributes on the ServerProxy
-           without interfering with the magic __getattr__
-        """
-        if attr == "close":
-            return self.__close
-        elif attr == "transport":
-            return self.__transport
-        raise AttributeError("Attribute %r not found" % (attr,))
+if not hasattr(xmlrpc_client.Transport, 'single_request'):
+    SCGITransport.request = SCGITransport.single_request
 
 
-class RTorrent(object):
+def create_proxy(url):
+    parsed = urlsplit(url)
+    if not parsed.scheme:
+        path = parsed.path
+        return xmlrpc_client.ServerProxy('http://1', transport=SCGITransport(socket_path=path))
+    if parsed.scheme == 'scgi':
+        url = 'http://%s' % parsed.netloc
+        return xmlrpc_client.ServerProxy(url, transport=SCGITransport())
+    logger.debug('Creating Normal XMLRPC Proxy with url {!r}', url)
+    return xmlrpc_client.ServerProxy(url)
+
+
+class RTorrent:
     """ rTorrent API client """
 
     default_fields = (
         'hash',
         'name',
-        'up_total', 'down_total', 'down_rate',
-        'is_open', 'is_active',
-        'custom1', 'custom2', 'custom3', 'custom4', 'custom5',
-        'state', 'complete',
-        'bytes_done', 'down.rate', 'left_bytes',
+        'up_total',
+        'down_total',
+        'down_rate',
+        'is_open',
+        'is_active',
+        'custom1',
+        'custom2',
+        'custom3',
+        'custom4',
+        'custom5',
+        'state',
+        'complete',
+        'bytes_done',
+        'down.rate',
+        'left_bytes',
         'ratio',
-        'base_path', 'load_date'
+        'base_path',
+        'load_date',
     )
 
-    required_fields = (
-        'hash',
-        'name',
-        'base_path'
-    )
+    required_fields = ('hash', 'name', 'base_path')
 
     def __init__(self, uri, username=None, password=None, digest_auth=None, session=None):
         """
@@ -261,23 +205,27 @@ class RTorrent(object):
         parsed_uri = urlparse(uri)
 
         if self.username and self.password and parsed_uri.scheme not in ['http', 'https']:
-            raise IOError('Username and password only supported on http(s)')
+            raise OSError('Username and password only supported on http(s)')
 
         # Determine the proxy server
         if parsed_uri.scheme in ['http', 'https']:
             sp = xmlrpc_client.ServerProxy
         elif parsed_uri.scheme == 'scgi':
-            sp = SCGIServerProxy
+            sp = create_proxy
         elif parsed_uri.scheme == '' and parsed_uri.path:
             self.uri = parsed_uri.path
-            sp = SCGIServerProxy
+            sp = create_proxy
         else:
-            raise IOError('Unsupported scheme %s for uri %s' % (parsed_uri.scheme, self.uri))
+            raise OSError('Unsupported scheme %s for uri %s' % (parsed_uri.scheme, self.uri))
 
         # Use a special transport if http(s)
         if parsed_uri.scheme in ['http', 'https']:
-            self._server = sp(self.uri, transport=HTTPDigestTransport(parsed_uri.scheme, self.digest_auth,
-                                                                      self.username, self.password, session))
+            self._server = sp(
+                self.uri,
+                transport=HTTPDigestTransport(
+                    parsed_uri.scheme, self.digest_auth, self.username, self.password, session
+                ),
+            )
         else:
             self._server = sp(self.uri)
 
@@ -288,7 +236,7 @@ class RTorrent(object):
         if reverse:
             for field in ['up.total', 'down.total', 'down.rate']:
                 if field in fields:
-                    fields[fields.index(field)] = native_str(field.replace('.', '_'))
+                    fields[fields.index(field)] = field.replace('.', '_')
             return fields
 
         for required_field in self.required_fields:
@@ -297,7 +245,7 @@ class RTorrent(object):
 
         for field in ['up_total', 'down_total', 'down_rate']:
             if field in fields:
-                fields[fields.index(field)] = native_str(field.replace('_', '.'))
+                fields[fields.index(field)] = field.replace('_', '.')
 
         return fields
 
@@ -311,7 +259,8 @@ class RTorrent(object):
         # Additional fields to set
         for key, val in fields.items():
             # Values must be escaped if within params
-            params.append('d.%s.set=%s' % (key, re.escape(native_str(val))))
+            # TODO: What are the escaping requirements? re.escape works differently on python 3.7+
+            params.append('d.%s.set=%s' % (key, re.escape(str(val))))
 
         if mkdir and 'directory' in fields:
             result = self._server.execute.throw('', 'mkdir', '-p', fields['directory'])
@@ -319,7 +268,9 @@ class RTorrent(object):
                 raise xmlrpc_client.Error('Failed creating directory %s' % fields['directory'])
 
         # by default rtorrent won't allow calls over 512kb in size.
-        xmlrpc_size = len(xmlrpc_client.dumps(tuple(params), 'raw_start')) + 71680  # Add 70kb for buffer
+        xmlrpc_size = (
+            len(xmlrpc_client.dumps(tuple(params), 'raw_start')) + 71680
+        )  # Add 70kb for buffer
         if xmlrpc_size > 524288:
             prev_size = self._server.network.xmlrpc.size_limit()
             self._server.network.xmlrpc.size_limit.set('', xmlrpc_size)
@@ -340,7 +291,6 @@ class RTorrent(object):
 
     def torrent(self, info_hash, fields=None):
         """ Get the details of a torrent """
-        info_hash = native_str(info_hash)
         if not fields:
             fields = list(self.default_fields)
 
@@ -364,7 +314,7 @@ class RTorrent(object):
         params = ['d.%s=' % field for field in fields]
         params.insert(0, view)
 
-        resp = self._server.d.multicall(params)
+        resp = self._server.d.multicall2('', params)
 
         # Response is formatted as a list of lists, with just the values
         return [dict(list(zip(self._clean_fields(fields, reverse=True), val))) for val in resp]
@@ -374,28 +324,27 @@ class RTorrent(object):
 
         for key, val in fields.items():
             method_name = 'd.%s.set' % key
-            getattr(multi_call, method_name)(native_str(info_hash), native_str(val))
+            getattr(multi_call, method_name)(info_hash, val)
 
         return multi_call()[0]
 
     def delete(self, info_hash):
-        return self._server.d.erase(native_str(info_hash))
+        return self._server.d.erase(info_hash)
 
     def stop(self, info_hash):
         self._server.d.stop(info_hash)
-        return self._server.d.close(native_str(info_hash))
+        return self._server.d.close(info_hash)
 
     def start(self, info_hash):
-        return self._server.d.start(native_str(info_hash))
+        return self._server.d.start(info_hash)
 
     def move(self, info_hash, dst_path):
-        info_hash = native_str(info_hash)
         self.stop(info_hash)
 
         torrent = self.torrent(info_hash, fields=['base_path'])
 
         try:
-            log.verbose('Creating destination directory `%s`' % dst_path)
+            logger.verbose('Creating destination directory `{}`', dst_path)
             self._server.execute.throw('', 'mkdir', '-p', dst_path)
         except xmlrpc_client.Error:
             raise xmlrpc_client.Error("unable to create folder %s" % dst_path)
@@ -405,18 +354,22 @@ class RTorrent(object):
         self.start(info_hash)
 
 
-class RTorrentPluginBase(object):
-    priority_map = {
-        'high': 3,
-        'medium': 2,
-        'low': 1,
-        'off': 0,
-    }
+class RTorrentPluginBase:
+    priority_map = {'high': 3, 'medium': 2, 'low': 1, 'off': 0}
 
     def _build_options(self, config, entry, entry_first=True):
         options = {}
 
-        for opt_key in ('path', 'message', 'priority', 'custom1', 'custom2', 'custom3', 'custom4', 'custom5'):
+        for opt_key in (
+            'path',
+            'message',
+            'priority',
+            'custom1',
+            'custom2',
+            'custom3',
+            'custom4',
+            'custom5',
+        ):
             # Values do not merge config with task
             # Task takes priority then config is used
             entry_value = entry.get(opt_key)
@@ -470,7 +423,7 @@ class RTorrentOutputPlugin(RTorrentPluginBase):
             'custom3': {'type': 'string'},
             'custom4': {'type': 'string'},
             'custom5': {'type': 'string'},
-            'fast_resume': {'type': 'boolean', 'default': False}
+            'fast_resume': {'type': 'boolean', 'default': False},
         },
         'required': ['uri'],
         'additionalProperties': False,
@@ -491,23 +444,25 @@ class RTorrentOutputPlugin(RTorrentPluginBase):
         # If the download plugin is not enabled, we need to call it to get
         # our temp .torrent files
         if config['action'] == 'add' and 'download' not in task.config:
-            download = plugin.get_plugin_by_name('download')
-            download.instance.get_temp_files(task, handle_magnets=True, fail_html=True)
+            download = plugin.get('download', self)
+            download.get_temp_files(task, handle_magnets=True, fail_html=True)
 
     @plugin.priority(135)
     def on_task_output(self, task, config):
 
-        client = RTorrent(os.path.expanduser(config['uri']),
-                          username=config.get('username'),
-                          password=config.get('password'),
-                          digest_auth=config['digest_auth'],
-                          session=task.requests)
+        client = RTorrent(
+            os.path.expanduser(config['uri']),
+            username=config.get('username'),
+            password=config.get('password'),
+            digest_auth=config['digest_auth'],
+            session=task.requests,
+        )
 
         try:
             for entry in task.accepted:
                 if config['action'] == 'add':
                     if task.options.test:
-                        log.info('Would add %s to rTorrent', entry['url'])
+                        logger.info('Would add {} to rTorrent', entry['url'])
                         continue
                     try:
                         options = self._build_options(config, entry)
@@ -517,8 +472,14 @@ class RTorrentOutputPlugin(RTorrentPluginBase):
 
                     # fast_resume is not really an rtorrent option so it's not in _build_options
                     fast_resume = entry.get('fast_resume', config['fast_resume'])
-                    self.add_entry(client, entry, options, start=config['start'], mkdir=config['mkdir'],
-                                   fast_resume=fast_resume)
+                    self.add_entry(
+                        client,
+                        entry,
+                        options,
+                        start=config['start'],
+                        mkdir=config['mkdir'],
+                        fast_resume=fast_resume,
+                    )
 
                 info_hash = entry.get('torrent_info_hash')
 
@@ -528,23 +489,33 @@ class RTorrentOutputPlugin(RTorrentPluginBase):
 
                 if config['action'] == 'delete':
                     if task.options.test:
-                        log.info('Would delete %s (%s) from rTorrent', entry['title'], entry['torrent_info_hash'])
+                        logger.info(
+                            'Would delete {} ({}) from rTorrent',
+                            entry['title'],
+                            entry['torrent_info_hash'],
+                        )
                         continue
                     self.delete_entry(client, entry)
 
                 if config['action'] == 'update':
                     if task.options.test:
-                        log.info('Would update %s (%s) in rTorrent', entry['title'], entry['torrent_info_hash'])
+                        logger.info(
+                            'Would update {} ({}) in rTorrent',
+                            entry['title'],
+                            entry['torrent_info_hash'],
+                        )
                         continue
                     self.update_entry(client, entry, config)
 
-        except IOError as e:
+        except OSError as e:
             raise plugin.PluginError("Couldn't connect to rTorrent: %s" % str(e))
 
     def delete_entry(self, client, entry):
         try:
             client.delete(entry['torrent_info_hash'])
-            log.verbose('Deleted %s (%s) in rtorrent ' % (entry['title'], entry['torrent_info_hash']))
+            logger.verbose(
+                'Deleted {} ({}) in rtorrent ', entry['title'], entry['torrent_info_hash']
+            )
         except xmlrpc_client.Error as e:
             entry.fail('Failed to delete: %s' % str(e))
             return
@@ -567,11 +538,15 @@ class RTorrentOutputPlugin(RTorrentPluginBase):
 
         if existing and 'directory' in options:
             # Check if changing to another directory which requires a move
-            if options['directory'] != existing['base_path'] \
-                    and options['directory'] != os.path.dirname(existing['base_path']):
+            if options['directory'] != existing['base_path'] and options[
+                'directory'
+            ] != os.path.dirname(existing['base_path']):
                 try:
-                    log.verbose("Path is changing, moving files from '%s' to '%s'"
-                                % (existing['base_path'], options['directory']))
+                    logger.verbose(
+                        "Path is changing, moving files from '{}' to '{}'",
+                        existing['base_path'],
+                        options['directory'],
+                    )
                     client.move(info_hash, options['directory'])
                 except xmlrpc_client.Error as e:
                     entry.fail('Failed moving torrent: %s' % str(e))
@@ -583,7 +558,7 @@ class RTorrentOutputPlugin(RTorrentPluginBase):
 
         try:
             client.update(info_hash, options)
-            log.verbose('Updated %s (%s) in rtorrent ' % (entry['title'], info_hash))
+            logger.verbose('Updated {} ({}) in rtorrent ', entry['title'], info_hash)
         except xmlrpc_client.Error as e:
             entry.fail('Failed to update: %s' % str(e))
             return
@@ -624,7 +599,9 @@ class RTorrentOutputPlugin(RTorrentPluginBase):
                 for f in entry['torrent'].get_filelist():
                     relative_file_path = os.path.join(f['path'], f['name'])
                     if entry['torrent'].is_multi_file:
-                        relative_file_path = os.path.join(entry['torrent'].name, relative_file_path)
+                        relative_file_path = os.path.join(
+                            entry['torrent'].name, relative_file_path
+                        )
                     file_path = os.path.join(base, relative_file_path)
                     # TODO should it simply add the torrent anyway?
                     if not os.path.exists(file_path) and not os.path.isfile(file_path):
@@ -642,7 +619,7 @@ class RTorrentOutputPlugin(RTorrentPluginBase):
             try:
                 with open(entry['file'], 'rb') as f:
                     torrent_raw = f.read()
-            except IOError as e:
+            except OSError as e:
                 entry.fail('Failed to add to rTorrent %s' % str(e))
                 return
 
@@ -655,7 +632,7 @@ class RTorrentOutputPlugin(RTorrentPluginBase):
         # First check if it already exists
         try:
             if client.torrent(entry['torrent_info_hash']):
-                log.warning("Torrent %s already exists, won't add" % entry['title'])
+                logger.warning("Torrent {} already exists, won't add", entry['title'])
                 return
         except xmlrpc_client.Error:
             # No existing found
@@ -666,23 +643,23 @@ class RTorrentOutputPlugin(RTorrentPluginBase):
             if resp != 0:
                 entry.fail('Failed to add to rTorrent invalid return value %s' % resp)
         except xmlrpc_client.Error as e:
-            log.exception(e)
+            logger.exception(e)
             entry.fail('Failed to add to rTorrent %s' % str(e))
             return
 
         # Verify the torrent loaded
         try:
             self._verify_load(client, entry['torrent_info_hash'])
-            log.info('%s added to rtorrent' % entry['title'])
+            logger.info('{} added to rtorrent', entry['title'])
         except xmlrpc_client.Error as e:
-            entry.fail('Failed to verify torrent loaded: %s' % str(e))
+            logger.warning('Failed to verify torrent {} loaded: {}', entry['title'], str(e))
 
     def on_task_learn(self, task, config):
         """ Make sure all temp files are cleaned up when entries are learned """
         # If download plugin is enabled, it will handle cleanup.
         if 'download' not in task.config:
-            download = plugin.get_plugin_by_name('download')
-            download.instance.cleanup_temp_files(task)
+            download = plugin.get('download', self)
+            download.cleanup_temp_files(task)
 
     on_task_abort = on_task_learn
 
@@ -699,21 +676,23 @@ class RTorrentInputPlugin(RTorrentPluginBase):
             'fields': one_or_more({'type': 'string', 'enum': list(RTorrent.default_fields)}),
         },
         'required': ['uri'],
-        'additionalProperties': False
+        'additionalProperties': False,
     }
 
     def on_task_input(self, task, config):
-        client = RTorrent(os.path.expanduser(config['uri']),
-                          username=config.get('username'),
-                          password=config.get('password'),
-                          digest_auth=config['digest_auth'],
-                          session=task.requests)
+        client = RTorrent(
+            os.path.expanduser(config['uri']),
+            username=config.get('username'),
+            password=config.get('password'),
+            digest_auth=config['digest_auth'],
+            session=task.requests,
+        )
 
         fields = config.get('fields')
 
         try:
             torrents = client.torrents(config['view'], fields=fields)
-        except (IOError, xmlrpc_client.Error) as e:
+        except (OSError, xmlrpc_client.Error) as e:
             task.abort('Could not get torrents (%s): %s' % (config['view'], e))
             return
 
@@ -722,8 +701,7 @@ class RTorrentInputPlugin(RTorrentPluginBase):
         for torrent in torrents:
             entry = Entry(
                 title=torrent['name'],
-                url='%s/%s' % (os.path.expanduser(config['uri']),
-                               torrent['hash']),
+                url='%s/%s' % (os.path.expanduser(config['uri']), torrent['hash']),
                 path=torrent['base_path'],
                 torrent_info_hash=torrent['hash'],
             )
