@@ -1,15 +1,12 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
-import logging
+from loguru import logger
 
 from flexget import plugin
 from flexget.event import event
 
-log = logging.getLogger('accept_all')
+logger = logger.bind(name='accept_all')
 
 
-class FilterAcceptAll(object):
+class FilterAcceptAll:
     """
         Just accepts all entries.
 
