@@ -1,16 +1,13 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
-import logging
+from loguru import logger
 
 from flexget import plugin
 from flexget.event import event
 from flexget.utils.tools import aggregate_inputs
 
-log = logging.getLogger('crossmatch')
+logger = logger.bind(name='crossmatch')
 
 
-class CrossMatch(object):
+class CrossMatch:
     """
     Perform action based on item on current task and other inputs.
 
@@ -23,6 +20,7 @@ class CrossMatch(object):
           - title
         action: reject
         exact: yes
+        case_sensitive: yes
     """
 
     schema = {
@@ -32,11 +30,11 @@ class CrossMatch(object):
             'action': {'enum': ['accept', 'reject']},
             'from': {'type': 'array', 'items': {'$ref': '/schema/plugins?phase=input'}},
             'exact': {'type': 'boolean', 'default': True},
-            'all_fields': {'type': 'boolean', 'default': False}
-
+            'all_fields': {'type': 'boolean', 'default': False},
+            'case_sensitive': {'type': 'boolean', 'default': True},
         },
         'required': ['fields', 'action', 'from'],
-        'additionalProperties': False
+        'additionalProperties': False,
     }
 
     def on_task_filter(self, task, config):
@@ -45,15 +43,28 @@ class CrossMatch(object):
         action = config['action']
         all_fields = config['all_fields']
 
+        if not task.entries:
+          logger.trace('Stopping crossmatch filter because of no entries to check')
+          return
+        
         match_entries = aggregate_inputs(task, config['from'])
 
         # perform action on intersecting entries
         for entry in task.entries:
             for generated_entry in match_entries:
-                log.trace('checking if %s matches %s', entry['title'], generated_entry['title'])
-                common = self.entry_intersects(entry, generated_entry, fields, config.get('exact'))
+                logger.trace('checking if {} matches {}', entry['title'], generated_entry['title'])
+                common = self.entry_intersects(
+                    entry,
+                    generated_entry,
+                    fields,
+                    config.get('exact'),
+                    config.get('case_sensitive'),
+                )
                 if common and (not all_fields or len(common) == len(fields)):
-                    msg = 'intersects with %s on field(s) %s' % (generated_entry['title'], ', '.join(common))
+                    msg = 'intersects with %s on field(s) %s' % (
+                        generated_entry['title'],
+                        ', '.join(common),
+                    )
                     for key in generated_entry:
                         if key not in entry:
                             entry[key] = generated_entry[key]
@@ -62,7 +73,7 @@ class CrossMatch(object):
                     if action == 'accept':
                         entry.accept(msg)
 
-    def entry_intersects(self, e1, e2, fields=None, exact=True):
+    def entry_intersects(self, e1, e2, fields=None, exact=True, case_sensitive=True):
         """
         :param e1: First :class:`flexget.entry.Entry`
         :param e2: Second :class:`flexget.entry.Entry`
@@ -78,20 +89,26 @@ class CrossMatch(object):
         for field in fields:
             # Doesn't really make sense to match if field is not in both entries
             if field not in e1 or field not in e2:
-                log.trace('field %s is not in both entries', field)
+                logger.trace('field {} is not in both entries', field)
                 continue
 
-            v1 = e1[field]
-            v2 = e2[field]
+            if not case_sensitive and isinstance(e1[field], str):
+                v1 = e1[field].lower()
+            else:
+                v1 = e1[field]
+            if not case_sensitive and isinstance(e1[field], str):
+                v2 = e2[field].lower()
+            else:
+                v2 = e2[field]
 
             try:
                 if v1 == v2 or not exact and (v2 in v1 or v1 in v2):
                     common_fields.append(field)
                 else:
-                    log.trace('not matching')
+                    logger.trace('not matching')
             except TypeError as e:
                 # argument of type <type> is not iterable
-                log.trace('error matching fields: %s', str(e))
+                logger.trace('error matching fields: {}', str(e))
 
         return common_fields
 
