@@ -1,17 +1,14 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
-import logging
+from loguru import logger
 
 from flexget import plugin
-from flexget.event import event
 from flexget.config_schema import one_or_more
+from flexget.event import event
 from flexget.utils import qualities
 
-log = logging.getLogger('quality')
+logger = logger.bind(name='quality')
 
 
-class FilterQuality(object):
+class FilterQuality:
     """
     Rejects all entries that don't have one of the specified qualities
 
