@@ -1,16 +1,13 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
-import logging
+from loguru import logger
 
 from flexget import plugin
 from flexget.event import event
 from flexget.utils.log import log_once
 
-log = logging.getLogger('rt')
+logger = logger.bind(name='rt')
 
 
-class FilterRottenTomatoes(object):
+class FilterRottenTomatoes:
     """
         This plugin allows filtering based on Rotten Tomatoes score, votes and genres etc.
 
@@ -61,39 +58,37 @@ class FilterRottenTomatoes(object):
             - G
     """
 
-    def __init__(self):
-        # We could pull these from the API through lists.json but that's extra web/API key usage
-        self.critics_ratings = {'rotten': 0, 'fresh': 1, 'certified fresh': 2}
-        self.audience_ratings = {'spilled': 0, 'upright': 1}
+    critics_ratings = {'rotten': 0, 'fresh': 1, 'certified fresh': 2}
+    audience_ratings = {'spilled': 0, 'upright': 1}
 
-    def validator(self):
-        from flexget import validator
-        rt = validator.factory('dict')
-        rt.accept('integer', key='min_year')
-        rt.accept('integer', key='max_year')
-        rt.accept('number', key='min_critics_score')
-        rt.accept('number', key='min_audience_score')
-        rt.accept('number', key='min_average_score')
-        rt.accept('choice', key='min_critics_rating').accept_choices(list(self.critics_ratings.keys()))
-        rt.accept('choice', key='min_audience_rating').accept_choices(list(self.audience_ratings.keys()))
-        rt.accept('list', key='reject_genres').accept('text')
-        rt.accept('list', key='reject_actors').accept('text')
-        rt.accept('list', key='accept_actors').accept('text')
-        rt.accept('list', key='reject_directors').accept('text')
-        rt.accept('list', key='accept_directors').accept('text')
-        rt.accept('list', key='reject_mpaa_ratings').accept('text')
-        rt.accept('list', key='accept_mpaa_ratings').accept('text')
-        return rt
+    schema = {
+        'type': 'object',
+        'properties': {
+            'min_year': {'type': 'integer'},
+            'max_year': {'type': 'integer'},
+            'min_critics_score': {'type': 'number'},
+            'min_audience_score': {'type': 'number'},
+            'min_average_score': {'type': 'number'},
+            'min_critics_rating': {'enum': list(critics_ratings)},
+            'min_audience_rating': {'enum': list(audience_ratings)},
+            'reject_genres': {'type': 'array', 'items': {'type': 'string'}},
+            'reject_actors': {'type': 'array', 'items': {'type': 'string'}},
+            'accept_actors': {'type': 'array', 'items': {'type': 'string'}},
+            'reject_directors': {'type': 'array', 'items': {'type': 'string'}},
+            'accept_directors': {'type': 'array', 'items': {'type': 'string'}},
+            'reject_mpaa_ratings': {'type': 'array', 'items': {'type': 'string'}},
+            'accept_mpaa_ratings': {'type': 'array', 'items': {'type': 'string'}},
+        },
+        'additionalProperties': False,
+    }
 
     # Run later to avoid unnecessary lookups
     @plugin.priority(115)
     def on_task_filter(self, task, config):
-
-        lookup = plugin.get_plugin_by_name('rottentomatoes_lookup').instance.lookup
+        lookup = plugin.get('rottentomatoes_lookup', self).lookup
 
         # since the plugin does not reject anything, no sense going trough accepted
         for entry in task.undecided:
-
             force_accept = False
 
             try:
@@ -101,8 +96,7 @@ class FilterRottenTomatoes(object):
             except plugin.PluginError as e:
                 # logs skip message once through log_once (info) and then only when ran from cmd line (w/o --cron)
                 msg = 'Skipping %s because of an error: %s' % (entry['title'], e.value)
-                if not log_once(msg, logger=log):
-                    log.verbose(msg)
+                log_once(msg, logger=logger)
                 continue
 
             # for key, value in entry.iteritems():
@@ -112,36 +106,54 @@ class FilterRottenTomatoes(object):
             reasons = []
             if 'min_critics_score' in config:
                 if entry.get('rt_critics_score', 0) < config['min_critics_score']:
-                    reasons.append('min_critics_score (%s < %s)' % (entry.get('rt_critics_score'),
-                                                                    config['min_critics_score']))
+                    reasons.append(
+                        'min_critics_score (%s < %s)'
+                        % (entry.get('rt_critics_score'), config['min_critics_score'])
+                    )
             if 'min_audience_score' in config:
                 if entry.get('rt_audience_score', 0) < config['min_audience_score']:
-                    reasons.append('min_audience_score (%s < %s)' % (entry.get('rt_audience_score'),
-                                                                     config['min_audience_score']))
+                    reasons.append(
+                        'min_audience_score (%s < %s)'
+                        % (entry.get('rt_audience_score'), config['min_audience_score'])
+                    )
             if 'min_average_score' in config:
                 if entry.get('rt_average_score', 0) < config['min_average_score']:
-                    reasons.append('min_average_score (%s < %s)' % (entry.get('rt_average_score'),
-                                                                    config['min_average_score']))
+                    reasons.append(
+                        'min_average_score (%s < %s)'
+                        % (entry.get('rt_average_score'), config['min_average_score'])
+                    )
             if 'min_critics_rating' in config:
                 if not entry.get('rt_critics_rating'):
                     reasons.append('min_critics_rating (no rt_critics_rating)')
-                elif self.critics_ratings.get(entry.get('rt_critics_rating').lower(),
-                                              0) < self.critics_ratings[config['min_critics_rating']]:
-                    reasons.append('min_critics_rating (%s < %s)' % (entry.get('rt_critics_rating').lower(),
-                                                                     config['min_critics_rating']))
+                elif (
+                    self.critics_ratings.get(entry.get('rt_critics_rating').lower(), 0)
+                    < self.critics_ratings[config['min_critics_rating']]
+                ):
+                    reasons.append(
+                        'min_critics_rating (%s < %s)'
+                        % (entry.get('rt_critics_rating').lower(), config['min_critics_rating'])
+                    )
             if 'min_audience_rating' in config:
                 if not entry.get('rt_audience_rating'):
                     reasons.append('min_audience_rating (no rt_audience_rating)')
-                elif self.audience_ratings.get(entry.get('rt_audience_rating').lower(),
-                                               0) < self.audience_ratings[config['min_audience_rating']]:
-                    reasons.append('min_audience_rating (%s < %s)' % (entry.get('rt_audience_rating').lower(),
-                                                                      config['min_audience_rating']))
+                elif (
+                    self.audience_ratings.get(entry.get('rt_audience_rating').lower(), 0)
+                    < self.audience_ratings[config['min_audience_rating']]
+                ):
+                    reasons.append(
+                        'min_audience_rating (%s < %s)'
+                        % (entry.get('rt_audience_rating').lower(), config['min_audience_rating'])
+                    )
             if 'min_year' in config:
                 if entry.get('rt_year', 0) < config['min_year']:
-                    reasons.append('min_year (%s < %s)' % (entry.get('rt_year'), config['min_year']))
+                    reasons.append(
+                        'min_year (%s < %s)' % (entry.get('rt_year'), config['min_year'])
+                    )
             if 'max_year' in config:
                 if entry.get('rt_year', 0) > config['max_year']:
-                    reasons.append('max_year (%s > %s)' % (entry.get('rt_year'), config['max_year']))
+                    reasons.append(
+                        'max_year (%s > %s)' % (entry.get('rt_year'), config['max_year'])
+                    )
             if 'reject_genres' in config:
                 rejected = config['reject_genres']
                 for genre in entry.get('rt_genres', []):
@@ -161,7 +173,7 @@ class FilterRottenTomatoes(object):
                 accepted = config['accept_actors']
                 for actor_name in entry.get('rt_actors', []):
                     if actor_name in accepted:
-                        log.debug('Accepting because of accept_actors %s' % actor_name)
+                        logger.debug('Accepting because of accept_actors {}', actor_name)
                         force_accept = True
                         break
 
@@ -177,7 +189,7 @@ class FilterRottenTomatoes(object):
                 accepted = config['accept_directors']
                 for director_name in entry.get('rt_directors', []):
                     if director_name in accepted:
-                        log.debug('Accepting because of accept_directors %s' % director_name)
+                        logger.debug('Accepting because of accept_directors {}', director_name)
                         force_accept = True
                         break
 
@@ -192,17 +204,19 @@ class FilterRottenTomatoes(object):
                     reasons.append('accept_mpaa_ratings %s' % entry.get('rt_mpaa_rating'))
 
             if reasons and not force_accept:
-                msg = 'Didn\'t accept `%s` because of rule(s) %s' % \
-                      (entry.get('rt_name', None) or entry['title'], ', '.join(reasons))
+                msg = 'Didn\'t accept `%s` because of rule(s) %s' % (
+                    entry.get('rt_name', None) or entry['title'],
+                    ', '.join(reasons),
+                )
                 if task.options.debug:
-                    log.debug(msg)
+                    logger.debug(msg)
                 else:
                     if task.options.cron:
-                        log_once(msg, log)
+                        log_once(msg, logger)
                     else:
-                        log.info(msg)
+                        logger.info(msg)
             else:
-                log.debug('Accepting %s' % (entry['title']))
+                logger.debug('Accepting {}', entry['title'])
                 entry.accept()
 
 
