@@ -1,16 +1,13 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
-import logging
+from loguru import logger
 
 from flexget import plugin
 from flexget.event import event
 from flexget.utils.log import log_once
 
-log = logging.getLogger('thetvdb')
+logger = logger.bind(name='thetvdb')
 
 
-class FilterTvdb(object):
+class FilterTvdb:
     """
         This plugin allows filtering based on thetvdb series rating,
         episode rating, status, genres, runtime, content-rating,
@@ -103,10 +100,10 @@ class FilterTvdb(object):
             'accept_directors': {'type': 'array', 'items': {'type': 'string'}},
             'reject_directors': {'type': 'array', 'items': {'type': 'string'}},
         },
-        'additionalProperties': False
+        'additionalProperties': False,
     }
 
-    def is_in_set(self, config, configkey, entryitem, ):
+    def is_in_set(self, config, configkey, entryitem):
         '''
         this takes the config object, config key (to a list), and entry
         item so it can return True if the object matches,
@@ -129,7 +126,7 @@ class FilterTvdb(object):
     @plugin.priority(126)
     def on_task_filter(self, task, config):
 
-        lookup = plugin.get_plugin_by_name('thetvdb_lookup').instance.lookup
+        lookup = plugin.get('thetvdb_lookup', self).lookup
 
         for entry in task.entries:
             force_accept = False
@@ -137,25 +134,41 @@ class FilterTvdb(object):
             try:
                 lookup(task, entry)
             except plugin.PluginError as e:
-                log.error('Skipping %s because of an error: %s' % (entry['title'], e.value))
+                logger.error('Skipping {} because of an error: {}', entry['title'], e.value)
                 continue
 
             # Check defined conditions
             reasons = []
             if 'min_series_rating' in config:
                 if entry['tvdb_rating'] < config['min_series_rating']:
-                    reasons.append('series_rating (%s < %s)' % (entry['tvdb_rating'], config['min_series_rating']))
+                    reasons.append(
+                        'series_rating (%s < %s)'
+                        % (entry['tvdb_rating'], config['min_series_rating'])
+                    )
             if 'min_episode_rating' in config:
                 if entry['tvdb_ep_rating'] < config['min_episode_rating']:
-                    reasons.append('tvdb_ep_rating (%s < %s)' % (entry['tvdb_ep_rating'], config['min_episode_rating']))
+                    reasons.append(
+                        'tvdb_ep_rating (%s < %s)'
+                        % (entry['tvdb_ep_rating'], config['min_episode_rating'])
+                    )
             if 'min_episode_air_year' in config:
                 if entry['tvdb_ep_air_date'].strftime("%Y") < config['min_episode_air_year']:
-                    reasons.append('tvdb_ep_air_date (%s < %s)' % (entry['tvdb_ep_air_date'].strftime("%Y"),
-                                                                   config['min_episode_air_year']))
+                    reasons.append(
+                        'tvdb_ep_air_date (%s < %s)'
+                        % (
+                            entry['tvdb_ep_air_date'].strftime("%Y"),
+                            config['min_episode_air_year'],
+                        )
+                    )
             if 'max_episode_air_year' in config:
                 if entry['tvdb_ep_air_date'].strftime("%Y") > config['max_episode_air_year']:
-                    reasons.append('tvdb_ep_air_date (%s < %s)' % (entry['tvdb_ep_air_date'].strftime("%Y"),
-                                                                   config['max_episode_air_year']))
+                    reasons.append(
+                        'tvdb_ep_air_date (%s < %s)'
+                        % (
+                            entry['tvdb_ep_air_date'].strftime("%Y"),
+                            config['max_episode_air_year'],
+                        )
+                    )
 
             if self.is_in_set(config, 'reject_content_rating', entry['tvdb_content_rating']):
                 reasons.append('reject_content_rating')
@@ -176,10 +189,14 @@ class FilterTvdb(object):
                 reasons.append('reject_status')
 
             # Accept if actors contains an accepted actor, but don't reject otherwise
-            if self.is_in_set(config, 'accept_actors', entry['tvdb_actors'] + entry['tvdb_ep_guest_stars']):
+            if self.is_in_set(
+                config, 'accept_actors', entry['tvdb_actors'] + entry['tvdb_ep_guest_stars']
+            ):
                 force_accept = True
 
-            if self.is_in_set(config, 'reject_actors', entry['tvdb_actors'] + entry['tvdb_ep_guest_stars']):
+            if self.is_in_set(
+                config, 'reject_actors', entry['tvdb_actors'] + entry['tvdb_ep_guest_stars']
+            ):
                 reasons.append('reject_genres')
 
             # Accept if director is an accepted director, but don't reject otherwise
@@ -190,14 +207,16 @@ class FilterTvdb(object):
                 reasons.append('reject_directors')
 
             if reasons and not force_accept:
-                msg = 'Skipping %s because of rule(s) %s' % \
-                      (entry.get('series_name_thetvdb', None) or entry['title'], ', '.join(reasons))
+                msg = 'Skipping %s because of rule(s) %s' % (
+                    entry.get('series_name_thetvdb', None) or entry['title'],
+                    ', '.join(reasons),
+                )
                 if task.options.debug:
-                    log.debug(msg)
+                    logger.debug(msg)
                 else:
-                    log_once(msg, log)
+                    log_once(msg, logger)
             else:
-                log.debug('Accepting %s' % (entry))
+                logger.debug('Accepting {}', entry)
                 entry.accept()
 
 
