@@ -1,11 +1,8 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
 from flexget import plugin
 from flexget.event import event
 
 
-class AbortPlugin(object):
+class AbortPlugin:
     """
     abort plugin for debug purposes.
 
@@ -13,10 +10,11 @@ class AbortPlugin(object):
 
         abort: yes
     """
+
     def on_task_output(self, task, config):
         task.abort('abort plugin')
 
 
 @event('plugin.register')
 def register():
-    plugin.register(AbortPlugin, 'abort', debug=True, api_ver=2)
\ No newline at end of file
+    plugin.register(AbortPlugin, 'abort', debug=True, api_ver=2)
