@@ -1,17 +1,15 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
 import datetime
-import logging
+
+from loguru import logger
 
 from flexget import options, plugin
 from flexget.config_schema import parse_interval
 from flexget.event import event
 
-log = logging.getLogger('interval')
+logger = logger.bind(name='interval')
 
 
-class PluginInterval(object):
+class PluginInterval:
     """
         Allows specifying minimum interval for task execution.
 
@@ -24,26 +22,28 @@ class PluginInterval(object):
 
     schema = {'type': 'string', 'format': 'interval'}
 
-    @plugin.priority(255)
+    @plugin.priority(plugin.PRIORITY_FIRST)
     def on_task_start(self, task, config):
         if task.options.learn:
-            log.info('Ignoring task %s interval for --learn' % task.name)
+            logger.info('Ignoring task {} interval for --learn', task.name)
             return
         last_time = task.simple_persistence.get('last_time')
         if not last_time:
-            log.info('No previous run recorded, running now')
+            logger.info('No previous run recorded, running now')
         elif task.options.interval_ignore:
-            log.info('Ignoring interval because of --now')
+            logger.info('Ignoring interval because of --now')
         else:
-            log.debug('last_time: %r' % last_time)
-            log.debug('interval: %s' % config)
+            logger.debug('last_time: {!r}', last_time)
+            logger.debug('interval: {}', config)
             next_time = last_time + parse_interval(config)
-            log.debug('next_time: %r' % next_time)
+            logger.debug('next_time: {!r}', next_time)
             if datetime.datetime.now() < next_time:
-                log.verbose('Interval %s not met on task %s. Use --now to override.' % (config, task.name))
+                logger.verbose(
+                    'Interval {} not met on task {}. Use --now to override.', config, task.name
+                )
                 task.abort('Interval not met', silent=True)
                 return
-        log.debug('interval passed')
+        logger.debug('interval passed')
         task.simple_persistence['last_time'] = datetime.datetime.now()
 
 
@@ -54,5 +54,10 @@ def register_plugin():
 
 @event('options.register')
 def register_parser_arguments():
-    options.get_parser('execute').add_argument('--now', action='store_true', dest='interval_ignore', default=False,
-                                               help='run task(s) even if the interval plugin would normally prevent it')
+    options.get_parser('execute').add_argument(
+        '--now',
+        action='store_true',
+        dest='interval_ignore',
+        default=False,
+        help='run task(s) even if the interval plugin would normally prevent it',
+    )
