@@ -1,23 +1,25 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
 import pytest
-from datetime import datetime, timedelta
-
-from flexget.plugins.filter.seen import SeenEntry
 
 from flexget.api.app import base_message
-from flexget.api.plugins.series import ObjectsContainer as OC
-from flexget.api.plugins.tvdb_lookup import ObjectsContainer as tvdb
-from flexget.api.plugins.tvmaze_lookup import ObjectsContainer as tvmaze
-from flexget.manager import Session
-from flexget.plugins.filter.series import (
-    Series, SeriesTask, Episode, EpisodeRelease, AlternateNames, Season, SeasonRelease
+from flexget.components.seen.db import SeenEntry
+from flexget.components.series.api import ObjectsContainer as OC
+# TODO: would be nicer to import db module
+from flexget.components.series.db import (
+    AlternateNames,
+    Episode,
+    EpisodeRelease,
+    Season,
+    SeasonRelease,
+    Series,
+    SeriesTask,
 )
+from flexget.components.thetvdb.api import ObjectsContainer as tvdb
+from flexget.components.tvmaze.api import ObjectsContainer as tvmaze
+from flexget.manager import Session
 from flexget.utils import json
 
 
-class TestSeriesRootAPI(object):
+class TestSeriesRootAPI:
     config = """
         tasks: {}
     """
@@ -245,8 +247,7 @@ class TestSeriesRootAPI(object):
         errors = schema_match(base_message, data)
         assert not errors
 
-        payload2 = {'name': 'test series 2',
-                    'begin_episode': 'bla'}
+        payload2 = {'name': 'test series 2', 'begin_episode': 'bla'}
 
         # Invalid begin episode
         rsp = api_client.json_post('/series/', data=json.dumps(payload2))
@@ -256,11 +257,11 @@ class TestSeriesRootAPI(object):
         errors = schema_match(base_message, data)
         assert not errors
 
-        payload3 = {'name': 'test series 2',
-                    'begin_episode': 's01e01',
-                    'alternate_names': [
-                        'show1', 'show2'
-                    ]}
+        payload3 = {
+            'name': 'test series 2',
+            'begin_episode': 's01e01',
+            'alternate_names': ['show1', 'show2'],
+        }
 
         # Maximal payload
         rsp = api_client.json_post('/series/', data=json.dumps(payload3))
@@ -274,8 +275,7 @@ class TestSeriesRootAPI(object):
         assert data['alternate_names'] == payload3['alternate_names']
         assert data['begin_episode']['identifier'].lower() == payload3['begin_episode']
 
-        payload4 = {'name': 'test series 3',
-                    'alternate_names': ['show1']}
+        payload4 = {'name': 'test series 3', 'alternate_names': ['show1']}
 
         # Alternate name already added to different show
         rsp = api_client.json_post('/series/', data=json.dumps(payload4))
@@ -286,7 +286,7 @@ class TestSeriesRootAPI(object):
         assert not errors
 
 
-class TestSeriesSearchAPI(object):
+class TestSeriesSearchAPI:
     config = """
         tasks: {}
     """
@@ -329,7 +329,7 @@ class TestSeriesSearchAPI(object):
         assert len(data) == 0
 
 
-class TestSeriesSingleAPI(object):
+class TestSeriesSingleAPI:
     config = """
         tasks: {}
     """
@@ -394,8 +394,7 @@ class TestSeriesSingleAPI(object):
         errors = schema_match(base_message, data)
         assert not errors
 
-        payload1 = {'begin_episode': 's01e01',
-                    'alternate_names': ['show1']}
+        payload1 = {'begin_episode': 's01e01', 'alternate_names': ['show1']}
 
         rsp = api_client.json_put('/series/1/', data=json.dumps(payload1))
         assert rsp.status_code == 200, 'Response code is %s' % rsp.status_code
@@ -434,7 +433,7 @@ class TestSeriesSingleAPI(object):
         assert not errors
 
 
-class TestSeriesSeasonsAPI(object):
+class TestSeriesSeasonsAPI:
     config = """
         tasks: {}
     """
@@ -504,7 +503,7 @@ class TestSeriesSeasonsAPI(object):
         assert len(data) == 0
 
 
-class TestSeriesSeasonAPI(object):
+class TestSeriesSeasonAPI:
     config = """
         tasks: {}
     """
@@ -608,7 +607,7 @@ class TestSeriesSeasonAPI(object):
         assert not errors
 
 
-class TestSeriesSeasonReleasesAPI(object):
+class TestSeriesSeasonReleasesAPI:
     config = """
         tasks: {}
     """
@@ -819,7 +818,7 @@ class TestSeriesSeasonReleasesAPI(object):
         assert not errors
 
 
-class TestSeriesEpisodesAPI(object):
+class TestSeriesEpisodesAPI:
     config = """
         tasks: {}
     """
@@ -891,7 +890,7 @@ class TestSeriesEpisodesAPI(object):
         assert len(data) == 0
 
 
-class TestSeriesEpisodeAPI(object):
+class TestSeriesEpisodeAPI:
     config = """
         tasks: {}
     """
@@ -1007,7 +1006,7 @@ class TestSeriesEpisodeAPI(object):
         assert not errors
 
 
-class TestSeriesEpisodeReleasesAPI(object):
+class TestSeriesEpisodeReleasesAPI:
     config = """
         tasks: {}
     """
@@ -1285,7 +1284,7 @@ class TestSeriesEpisodeReleasesAPI(object):
         assert not errors
 
 
-class TestSeriesEpisodeReleaseAPI(object):
+class TestSeriesEpisodeReleaseAPI:
     config = """
         tasks: {}
     """
@@ -1617,7 +1616,7 @@ class TestSeriesEpisodeReleaseAPI(object):
         assert not errors
 
 
-class TestSeriesForgetFlag(object):
+class TestSeriesForgetFlag:
     config = """
             tasks:
               series_data:
@@ -1766,7 +1765,7 @@ class TestSeriesForgetFlag(object):
             assert len(seen) == 3
 
 
-class TestSeriesPagination(object):
+class TestSeriesPagination:
     config = 'tasks: {}'
 
     def test_series_pagination(self, api_client, link_headers):
