@@ -1,38 +1,39 @@
 """Torrenting utils, mostly for handling bencoding and torrent files."""
 # Torrent decoding is a short fragment from effbot.org. Site copyright says:
 # Test scripts and other short code fragments can be considered as being in the public domain.
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-
 import binascii
 import functools
 import re
-import logging
 
-log = logging.getLogger('torrent')
+from loguru import logger
+
+logger = logger.bind(name='torrent')
 
 # Magic indicator used to quickly recognize torrent files
 TORRENT_RE = re.compile(br'^d\d{1,3}:')
 
 # List of all standard keys in a metafile
 # See http://packages.python.org/pyrocore/apidocs/pyrocore.util.metafile-module.html#METAFILE_STD_KEYS
-METAFILE_STD_KEYS = [i.split('.') for i in (
-    "announce",
-    "announce-list",  # BEP-0012
-    "comment",
-    "created by",
-    "creation date",
-    "encoding",
-    "info",
-    "info.length",
-    "info.name",
-    "info.piece length",
-    "info.pieces",
-    "info.private",
-    "info.files",
-    "info.files.length",
-    "info.files.path",
-)]
+METAFILE_STD_KEYS = [
+    i.split('.')
+    for i in (
+        "announce",
+        "announce-list",  # BEP-0012
+        "comment",
+        "created by",
+        "creation date",
+        "encoding",
+        "info",
+        "info.length",
+        "info.name",
+        "info.piece length",
+        "info.pieces",
+        "info.private",
+        "info.files",
+        "info.files.length",
+        "info.files.path",
+    )
+]
 
 
 def clean_meta(meta, including_info=False, logger=None):
@@ -85,12 +86,14 @@ def is_torrent_file(metafilepath):
 
     magic_marker = bool(TORRENT_RE.match(data))
     if not magic_marker:
-        log.trace('%s doesn\'t seem to be a torrent, got `%s` (hex)' % (metafilepath, binascii.hexlify(data)))
+        logger.trace(
+            "{} doesn't seem to be a torrent, got `{}` (hex)", metafilepath, binascii.hexlify(data)
+        )
 
     return bool(magic_marker)
 
 
-def tokenize(text, match=re.compile(b'([idel])|(\d+):|(-?\d+)').match):
+def tokenize(text, match=re.compile(br'([idel])|(\d+):|(-?\d+)').match):
     i = 0
     while i < len(text):
         m = match(text, i)
@@ -98,7 +101,7 @@ def tokenize(text, match=re.compile(b'([idel])|(\d+):|(-?\d+)').match):
         i = m.end()
         if m.lastindex == 2:
             yield b's'
-            yield text[i:i + int(s)]
+            yield text[i : i + int(s)]
             i += int(s)
         else:
             yield s
@@ -191,8 +194,9 @@ def bencode(data):
     raise TypeError('Unknown type for bencode: ' + str(type(data)))
 
 
-class Torrent(object):
+class Torrent:
     """Represents a torrent"""
+
     # string type used for keys, if this ever changes, stuff like "x in y"
     # gets broken unless you coerce to this type
     KEY_TYPE = str
@@ -212,27 +216,34 @@ class Torrent(object):
         self.modified = False
 
     def __repr__(self):
-        return "%s(%s, %s)" % (self.__class__.__name__,
-                               ", ".join("%s=%r" % (key, self.content["info"].get(key))
-                                         for key in ("name", "length", "private",)),
-                               ", ".join("%s=%r" % (key, self.content.get(key))
-                                         for key in ("announce", "comment",)))
+        return "%s(%s, %s)" % (
+            self.__class__.__name__,
+            ", ".join(
+                "%s=%r" % (key, self.content["info"].get(key))
+                for key in ("name", "length", "private")
+            ),
+            ", ".join("%s=%r" % (key, self.content.get(key)) for key in ("announce", "comment")),
+        )
 
     def get_filelist(self):
         """Return array containing fileinfo dictionaries (name, length, path)"""
         files = []
         if 'length' in self.content['info']:
             # single file torrent
-            t = {'name': self.content['info']['name'],
-                 'size': self.content['info']['length'],
-                 'path': ''}
+            if 'name.utf-8' in self.content['info']:
+                name = self.content['info']['name.utf-8']
+            else:
+                name = self.content['info']['name']
+            t = {'name': name, 'size': self.content['info']['length'], 'path': ''}
             files.append(t)
         else:
             # multifile torrent
             for item in self.content['info']['files']:
-                t = {'path': '/'.join(item['path'][:-1]),
-                     'name': item['path'][-1],
-                     'size': item['length']}
+                if 'path.utf-8' in item:
+                    path = item['path.utf-8']
+                else:
+                    path = item['path']
+                t = {'path': '/'.join(path[:-1]), 'name': path[-1], 'size': item['length']}
                 files.append(t)
 
         # Decode strings
@@ -244,9 +255,14 @@ class Torrent(object):
                         item[field] = item[field].decode(self.content.get('encoding', 'cp1252'))
                     except UnicodeError:
                         # Broken beyond anything reasonable
-                        fallback = item[field].decode('utf-8', 'replace').replace(u'\ufffd', '_')
-                        log.warning('%s=%r field in torrent %r is wrongly encoded, falling back to `%s`' %
-                                    (field, item[field], self.content['info']['name'], fallback))
+                        fallback = item[field].decode('utf-8', 'replace').replace('\ufffd', '_')
+                        logger.warning(
+                            '{}={!r} field in torrent {!r} is wrongly encoded, falling back to `{}`',
+                            field,
+                            item[field],
+                            self.content['info']['name'],
+                            fallback,
+                        )
                         item[field] = fallback
 
         return files
@@ -298,6 +314,7 @@ class Torrent(object):
     def info_hash(self):
         """Return Torrent info hash"""
         import hashlib
+
         hash = hashlib.sha1()
         info_data = encode_dictionary(self.content['info'])
         hash.update(info_data)
