@@ -1,18 +1,28 @@
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-from past.builtins import basestring
-
-import re
 import copy
-import logging
+import functools
+import re
+from typing import Dict, Iterator, List, Optional, Tuple, Union
 
-log = logging.getLogger('utils.qualities')
+from loguru import logger
+
+from flexget.utils.serialization import Serializer
+
+logger = logger.bind(name='utils.qualities')
 
 
-class QualityComponent(object):
+@functools.total_ordering
+class QualityComponent:
     """"""
 
-    def __init__(self, type, value, name, regexp=None, modifier=None, defaults=None):
+    def __init__(
+        self,
+        type: str,
+        value: int,
+        name: str,
+        regexp: Optional[str] = None,
+        modifier: Optional[int] = None,
+        defaults: Optional[List[str]] = None,
+    ) -> None:
         """
         :param type: Type of quality component. (resolution, source, codec, or audio)
         :param value: Value used to sort this component with others of like type.
@@ -33,9 +43,9 @@ class QualityComponent(object):
         # compile regexp
         if regexp is None:
             regexp = re.escape(name)
-        self.regexp = re.compile('(?<![^\W_])(' + regexp + ')(?![^\W_])', re.IGNORECASE)
+        self.regexp = re.compile(r'(?<![^\W_])(' + regexp + r')(?![^\W_])', re.IGNORECASE)
 
-    def matches(self, text):
+    def matches(self, text: str) -> Tuple[bool, str]:
         """Test if quality matches to text.
 
         :param string text: data te be tested against
@@ -47,7 +57,7 @@ class QualityComponent(object):
             return False, ""
         else:
             # remove matching part from the text
-            text = text[:match.start()] + text[match.end():]
+            text = text[: match.start()] + text[match.end() :]
         return True, text
 
     def __hash__(self):
@@ -57,7 +67,7 @@ class QualityComponent(object):
         return bool(self.value)
 
     def __eq__(self, other):
-        if isinstance(other, basestring):
+        if isinstance(other, str):
             other = _registry.get(other)
         if not isinstance(other, QualityComponent):
             raise TypeError('Cannot compare %r and %r' % (self, other))
@@ -66,11 +76,8 @@ class QualityComponent(object):
         else:
             raise TypeError('Cannot compare %s and %s' % (self.type, other.type))
 
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
     def __lt__(self, other):
-        if isinstance(other, basestring):
+        if isinstance(other, str):
             other = _registry.get(other)
         if not isinstance(other, QualityComponent):
             raise TypeError('Cannot compare %r and %r' % (self, other))
@@ -79,15 +86,6 @@ class QualityComponent(object):
         else:
             raise TypeError('Cannot compare %s and %s' % (self.type, other.type))
 
-    def __ge__(self, other):
-        return not self.__lt__(other)
-
-    def __le__(self, other):
-        return self.__lt__(other) or self.__eq__(other)
-
-    def __gt__(self, other):
-        return not self.__le__(other)
-
     def __add__(self, other):
         if not isinstance(other, int):
             raise TypeError()
@@ -124,10 +122,10 @@ _resolutions = [
     QualityComponent('resolution', 40, '576p', '576p?'),
     QualityComponent('resolution', 45, 'hr'),
     QualityComponent('resolution', 50, '720i'),
-    QualityComponent('resolution', 60, '720p', '(1280x)?720(p|hd)?x?(50)?'),
+    QualityComponent('resolution', 60, '720p', '(1280x)?720(p|hd)?x?([56]0)?'),
     QualityComponent('resolution', 70, '1080i'),
-    QualityComponent('resolution', 80, '1080p', '(1920x)?1080p?x?(50)?'),
-    QualityComponent('resolution', 90, '2160p', '((3840x)?2160p?x?(50)?)|4k')
+    QualityComponent('resolution', 80, '1080p', '(1920x)?1080p?x?([56]0)?'),
+    QualityComponent('resolution', 90, '2160p', '((3840x)?2160p?x?([56]0)?)|4k'),
 ]
 _sources = [
     QualityComponent('source', 10, 'workprint', modifier=-8),
@@ -135,20 +133,20 @@ _sources = [
     QualityComponent('source', 30, 'ts', '(?:hd)?ts|telesync', modifier=-6),
     QualityComponent('source', 40, 'tc', 'tc|telecine', modifier=-5),
     QualityComponent('source', 50, 'r5', 'r[2-8c]', modifier=-4),
-    QualityComponent('source', 60, 'hdrip', 'hd[\W_]?rip', modifier=-3),
-    QualityComponent('source', 70, 'ppvrip', 'ppv[\W_]?rip', modifier=-2),
+    QualityComponent('source', 60, 'hdrip', r'hd[\W_]?rip', modifier=-3),
+    QualityComponent('source', 70, 'ppvrip', r'ppv[\W_]?rip', modifier=-2),
     QualityComponent('source', 80, 'preair', modifier=-1),
-    QualityComponent('source', 90, 'tvrip', 'tv[\W_]?rip'),
-    QualityComponent('source', 100, 'dsr', 'dsr|ds[\W_]?rip'),
-    QualityComponent('source', 110, 'sdtv', '(?:[sp]dtv|dvb)(?:[\W_]?rip)?'),
-    QualityComponent('source', 120, 'dvdscr', '(?:(?:dvd|web)[\W_]?)?scr(?:eener)?', modifier=0),
+    QualityComponent('source', 90, 'tvrip', r'tv[\W_]?rip'),
+    QualityComponent('source', 100, 'dsr', r'dsr|ds[\W_]?rip'),
+    QualityComponent('source', 110, 'sdtv', r'(?:[sp]dtv|dvb)(?:[\W_]?rip)?'),
+    QualityComponent('source', 120, 'dvdscr', r'(?:(?:dvd|web)[\W_]?)?scr(?:eener)?', modifier=0),
     QualityComponent('source', 130, 'bdscr', 'bdscr(?:eener)?'),
-    QualityComponent('source', 140, 'webrip', 'web[\W_]?rip'),
-    QualityComponent('source', 150, 'hdtv', 'a?hdtv(?:[\W_]?rip)?'),
-    QualityComponent('source', 160, 'webdl', 'web(?:[\W_]?(dl|hd))?'),
-    QualityComponent('source', 170, 'dvdrip', 'dvd(?:[\W_]?rip)?'),
+    QualityComponent('source', 140, 'webrip', r'web[\W_]?rip'),
+    QualityComponent('source', 150, 'hdtv', r'a?hdtv(?:[\W_]?rip)?'),
+    QualityComponent('source', 160, 'webdl', r'web(?:[\W_]?(dl|hd))?'),
+    QualityComponent('source', 170, 'dvdrip', r'dvd(?:[\W_]?rip)?'),
     QualityComponent('source', 175, 'remux'),
-    QualityComponent('source', 180, 'bluray', '(?:b[dr][\W_]?rip|blu[\W_]?ray(?:[\W_]?rip)?)')
+    QualityComponent('source', 180, 'bluray', r'(?:b[dr][\W_]?rip|blu[\W_]?ray(?:[\W_]?rip)?)'),
 ]
 _codecs = [
     QualityComponent('codec', 10, 'divx'),
@@ -156,9 +154,9 @@ _codecs = [
     QualityComponent('codec', 30, 'h264', '[hx].?264'),
     QualityComponent('codec', 35, 'vp9'),
     QualityComponent('codec', 40, 'h265', '[hx].?265|hevc'),
-    QualityComponent('codec', 50, '10bit', '10.?bit|hi10p')
+    QualityComponent('codec', 50, '10bit', '10.?bit|hi10p'),
 ]
-channels = '(?:(?:[^\w+]?[1-7][\W_]?(?:0|1|ch)))'
+channels = r'(?:(?:[^\w+]?[1-7][\W_]?(?:0|1|ch)))'
 _audios = [
     QualityComponent('audio', 10, 'mp3'),
     # TODO: No idea what order these should go in or if we need different regexps
@@ -168,16 +166,16 @@ _audios = [
     QualityComponent('audio', 45, 'dd+5.1', 'dd[p+]%s' % channels),
     QualityComponent('audio', 50, 'flac', 'flac%s?' % channels),
     # The DTSs are a bit backwards, but the more specific one needs to be parsed first
-    QualityComponent('audio', 60, 'dtshd', 'dts[\W_]?hd(?:[\W_]?ma)?'),
+    QualityComponent('audio', 60, 'dtshd', r'dts[\W_]?hd(?:[\W_]?ma)?%s?' % channels),
     QualityComponent('audio', 70, 'dts'),
-    QualityComponent('audio', 80, 'truehd')
+    QualityComponent('audio', 80, 'truehd', 'truehd%s?' % channels),
 ]
 
 _UNKNOWNS = {
     'resolution': QualityComponent('resolution', 0, 'unknown'),
     'source': QualityComponent('source', 0, 'unknown'),
     'codec': QualityComponent('codec', 0, 'unknown'),
-    'audio': QualityComponent('audio', 0, 'unknown')
+    'audio': QualityComponent('audio', 0, 'unknown'),
 }
 
 # For wiki generating help
@@ -188,20 +186,21 @@ _UNKNOWNS = {
     print '}}}'
 '''
 
-_registry = {}
+_registry: Dict[str, QualityComponent] = {}
 for items in (_resolutions, _sources, _codecs, _audios):
     for item in items:
         _registry[item.name] = item
 
 
-def all_components():
+def all_components() -> Iterator[QualityComponent]:
     return iter(_registry.values())
 
 
-class Quality(object):
+@functools.total_ordering
+class Quality(Serializer):
     """Parses and stores the quality of an entry in the four component categories."""
 
-    def __init__(self, text=''):
+    def __init__(self, text: str = '') -> None:
         """
         :param text: A string to parse quality from
         """
@@ -215,7 +214,7 @@ class Quality(object):
             self.codec = _UNKNOWNS['codec']
             self.audio = _UNKNOWNS['audio']
 
-    def parse(self, text):
+    def parse(self, text: str):
         """Parses a string to determine the quality in the four component categories.
 
         :param text: The string to parse
@@ -233,7 +232,12 @@ class Quality(object):
                 if not getattr(self, default.type):
                     setattr(self, default.type, default)
 
-    def _find_best(self, qlist, default=None, strip_all=True):
+    def _find_best(
+        self,
+        qlist: List[QualityComponent],
+        default: Optional[QualityComponent] = None,
+        strip_all: bool = True,
+    ) -> QualityComponent:
         """Finds the highest matching quality component from `qlist`"""
         result = None
         search_in = self.clean_text
@@ -252,21 +256,31 @@ class Quality(object):
         return result or default
 
     @property
-    def name(self):
-        name = ' '.join(str(p) for p in (self.resolution, self.source, self.codec, self.audio) if p.value != 0)
+    def name(self) -> str:
+        name = ' '.join(
+            str(p) for p in (self.resolution, self.source, self.codec, self.audio) if p.value != 0
+        )
         return name or 'unknown'
 
     @property
-    def components(self):
+    def components(self) -> List[QualityComponent]:
         return [self.resolution, self.source, self.codec, self.audio]
 
+    @classmethod
+    def serialize(cls, quality: 'Quality') -> str:
+        return str(quality)
+
+    @classmethod
+    def deserialize(cls, data: str, version: int) -> 'Quality':
+        return cls(data)
+
     @property
-    def _comparator(self):
+    def _comparator(self) -> List:
         modifier = sum(c.modifier for c in self.components if c.modifier)
         return [modifier] + self.components
 
     def __contains__(self, other):
-        if isinstance(other, basestring):
+        if isinstance(other, str):
             other = Quality(other)
         if not other or not self:
             return False
@@ -280,40 +294,28 @@ class Quality(object):
         return any(self._comparator)
 
     def __eq__(self, other):
-        if isinstance(other, basestring):
+        if isinstance(other, str):
             other = Quality(other)
-            if not other:
-                raise TypeError('`%s` does not appear to be a valid quality string.' % other.text)
         if not isinstance(other, Quality):
             if other is None:
                 return False
             raise TypeError('Cannot compare %r and %r' % (self, other))
         return self._comparator == other._comparator
 
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
     def __lt__(self, other):
-        if isinstance(other, basestring):
+        if isinstance(other, str):
             other = Quality(other)
-            if not other:
-                raise TypeError('`%s` does not appear to be a valid quality string.' % other.text)
         if not isinstance(other, Quality):
             raise TypeError('Cannot compare %r and %r' % (self, other))
         return self._comparator < other._comparator
 
-    def __ge__(self, other):
-        return not self.__lt__(other)
-
-    def __le__(self, other):
-        return self.__lt__(other) or self.__eq__(other)
-
-    def __gt__(self, other):
-        return not self.__le__(other)
-
     def __repr__(self):
-        return '<Quality(resolution=%s,source=%s,codec=%s,audio=%s)>' % (self.resolution, self.source,
-                                                                         self.codec, self.audio)
+        return '<Quality(resolution=%s,source=%s,codec=%s,audio=%s)>' % (
+            self.resolution,
+            self.source,
+            self.codec,
+            self.audio,
+        )
 
     def __str__(self):
         return self.name
@@ -323,7 +325,7 @@ class Quality(object):
         return hash(self.name)
 
 
-def get(quality_name):
+def get(quality_name: str) -> Quality:
     """Returns a quality object based on canonical quality name."""
 
     found_components = {}
@@ -342,21 +344,21 @@ def get(quality_name):
     return result
 
 
-class RequirementComponent(object):
+class RequirementComponent:
     """Represents requirements for a given component type. Can evaluate whether a given QualityComponent
     meets those requirements."""
 
-    def __init__(self, type):
+    def __init__(self, type: str) -> None:
         self.type = type
         self.reset()
 
-    def reset(self):
+    def reset(self) -> None:
         self.min = None
         self.max = None
         self.acceptable = set()
         self.none_of = set()
 
-    def allows(self, comp, loose=False):
+    def allows(self, comp: QualityComponent, loose: bool = False) -> bool:
         if comp.type != self.type:
             raise TypeError('Cannot compare %r against %s' % (comp, self.type))
         if comp in self.none_of:
@@ -375,7 +377,7 @@ class RequirementComponent(object):
             return True
         return False
 
-    def add_requirement(self, text):
+    def add_requirement(self, text: str) -> None:
         if '-' in text:
             min, max = text.split('-')
             min, max = _registry[min], _registry[max]
@@ -407,17 +409,25 @@ class RequirementComponent(object):
                     self.none_of.add(qual)
 
     def __eq__(self, other):
-        return ((self.max, self.max, self.acceptable, self.none_of) ==
-                (other.max, other.max, other.acceptable, other.none_of))
+        return (self.max, self.max, self.acceptable, self.none_of) == (
+            other.max,
+            other.max,
+            other.acceptable,
+            other.none_of,
+        )
 
     def __hash__(self):
-        return hash(tuple([self.min, self.max, tuple(sorted(self.acceptable)), tuple(sorted(self.none_of))]))
+        return hash(
+            tuple(
+                [self.min, self.max, tuple(sorted(self.acceptable)), tuple(sorted(self.none_of))]
+            )
+        )
 
 
-class Requirements(object):
+class Requirements:
     """Represents requirements for allowable qualities. Can determine whether a given Quality passes requirements."""
 
-    def __init__(self, req=''):
+    def __init__(self, req: str = '') -> None:
         self.text = ''
         self.resolution = RequirementComponent('resolution')
         self.source = RequirementComponent('source')
@@ -427,10 +437,10 @@ class Requirements(object):
             self.parse_requirements(req)
 
     @property
-    def components(self):
+    def components(self) -> List[RequirementComponent]:
         return [self.resolution, self.source, self.codec, self.audio]
 
-    def parse_requirements(self, text):
+    def parse_requirements(self, text: str) -> None:
         """
         Parses a requirements string.
 
@@ -461,7 +471,7 @@ class Requirements(object):
         except KeyError as e:
             raise ValueError('%s is not a valid quality component.' % e.args[0])
 
-    def allows(self, qual, loose=False):
+    def allows(self, qual: Union[Quality, str], loose: bool = False) -> bool:
         """Determine whether this set of requirements allows a given quality.
 
         :param Quality qual: The quality to evaluate.
@@ -469,10 +479,8 @@ class Requirements(object):
         :rtype: bool
         :returns: True if given quality passes all component requirements.
         """
-        if isinstance(qual, basestring):
+        if isinstance(qual, str):
             qual = Quality(qual)
-            if not qual:
-                raise TypeError('`%s` does not appear to be a valid quality string.' % qual.text)
         for r_component, q_component in zip(self.components, qual.components):
             if not r_component.allows(q_component, loose=loose):
                 return False
