@@ -1,33 +1,44 @@
-from __future__ import unicode_literals, division, absolute_import
-from future.utils import text_to_native_str
-from flexget.utils.tools import native_str_to_text
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-from past.builtins import basestring
-
-import logging
-import os
-import re
 import locale
+import os
+import os.path
+import re
 from copy import copy
-from datetime import datetime, date, time
+from datetime import date, datetime, time
+from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Union
 
 import jinja2.filters
-from jinja2 import (Environment, StrictUndefined, ChoiceLoader, FileSystemLoader, PackageLoader, Template,
-                    TemplateNotFound, TemplateSyntaxError)
 from dateutil import parser as dateutil_parse
+from jinja2 import (
+    ChoiceLoader,
+    Environment,
+    FileSystemLoader,
+    PackageLoader,
+    StrictUndefined,
+    Template,
+    TemplateNotFound,
+    TemplateSyntaxError,
+)
+from jinja2.nativetypes import NativeTemplate
+from loguru import logger
 
 from flexget.event import event
 from flexget.utils.lazy_dict import LazyDict
 from flexget.utils.pathscrub import pathscrub
 
-log = logging.getLogger('utils.template')
+if TYPE_CHECKING:
+    from flexget.entry import Entry
+    from flexget.manager import Manager
+    from flexget.task import Task
+
+logger = logger.bind(name='utils.template')
 
 # The environment will be created after the manager has started
-environment = None
+environment: Optional[Environment] = None
 
 
 class RenderError(Exception):
     """Error raised when there is a problem with jinja rendering."""
+
     pass
 
 
@@ -63,7 +74,7 @@ def filter_re_replace(val, pattern, repl):
 
 def filter_re_search(val, pattern):
     """Perform a search for given regexp pattern, return the matching portion of the text."""
-    if not isinstance(val, basestring):
+    if not isinstance(val, str):
         return val
     result = re.search(pattern, val, re.IGNORECASE)
     if result:
@@ -76,7 +87,7 @@ def filter_formatdate(val, format):
     encoding = locale.getpreferredencoding()
     if not isinstance(val, (datetime, date, time)):
         return val
-    return native_str_to_text(val.strftime(text_to_native_str(format, encoding=encoding)), encoding=encoding)
+    return val.strftime(format)
 
 
 def filter_parsedate(val):
@@ -96,11 +107,11 @@ def filter_date_suffix(date):
 
 def filter_format_number(val, places=None, grouping=True):
     """Formats a number according to the user's locale."""
-    if not isinstance(val, (int, float, int)):
+    if not isinstance(val, (int, float)):
         return val
     if places is not None:
         format = '%.' + str(places) + 'f'
-    elif isinstance(val, (int, int)):
+    elif isinstance(val, int):
         format = '%d'
     else:
         format = '%.02f'
@@ -129,49 +140,78 @@ def filter_default(value, default_value='', boolean=True):
 filter_d = filter_default
 
 
-# TODO: In Jinja 2.8 we will be able to override the Context class to be used explicitly
+def is_fs_file(pathname):
+    """Test whether item is existing file in filesystem"""
+    return os.path.isfile(pathname)
+
+
+def is_fs_dir(pathname):
+    """Test whether item is existing directory in filesystem"""
+    return os.path.isdir(pathname)
+
+
+def is_fs_link(pathname):
+    """Test whether item is existing link in filesystem"""
+    return os.path.islink(pathname)
+
+
 class FlexGetTemplate(Template):
     """Adds lazy lookup support when rendering templates."""
 
     def new_context(self, vars=None, shared=False, locals=None):
-        context = super(FlexGetTemplate, self).new_context(vars, shared, locals)
+        context = super().new_context(vars, shared, locals)
         context.parent = LazyDict(context.parent)
         return context
 
 
+class FlexGetNativeTemplate(FlexGetTemplate, NativeTemplate):
+    """Lazy lookup support and native python return types."""
+
+    pass
+
+
 @event('manager.initialize')
-def make_environment(manager):
+def make_environment(manager: 'Manager') -> None:
     """Create our environment and add our custom filters"""
     global environment
-    environment = Environment(undefined=StrictUndefined,
-                              loader=ChoiceLoader([PackageLoader('flexget'),
-                                                   FileSystemLoader(os.path.join(manager.config_base, 'templates'))]),
-                              extensions=['jinja2.ext.loopcontrols'])
+    environment = Environment(
+        undefined=StrictUndefined,
+        loader=ChoiceLoader(
+            [
+                PackageLoader('flexget'),
+                FileSystemLoader(os.path.join(manager.config_base, 'templates')),
+            ]
+        ),
+        extensions=['jinja2.ext.loopcontrols'],
+    )
     environment.template_class = FlexGetTemplate
     for name, filt in list(globals().items()):
         if name.startswith('filter_'):
             environment.filters[name.split('_', 1)[1]] = filt
+    for name, test in list(globals().items()):
+        if name.startswith('is_'):
+            environment.tests[name.split('_', 1)[1]] = test
 
 
-def list_templates(extensions=None):
+def list_templates(extensions: Optional[List[str]] = None) -> List[str]:
     """
     Returns all templates names that are configured under environment loader dirs
     """
     if environment is None or not hasattr(environment, 'loader'):
-        return
+        return []
     return environment.list_templates(extensions=extensions)
 
 
-def get_filters():
+def get_filters() -> dict:
     """
     Returns all built-in and custom Jinja filters in a dict, where the key is the name, and the value is the filter func
     """
     if environment is None or not hasattr(environment, 'loader'):
         return {}
-    return environment.filters.items()
+    return environment.filters
 
 
-def get_template(template_name, scope='task'):
+def get_template(template_name: str, scope: Optional[str] = 'task') -> FlexGetTemplate:
     """Loads a template from disk. Looks in both included plugins and users custom scope dir."""
 
     if not template_name.endswith('.template'):
@@ -193,30 +233,36 @@ def get_template(template_name, scope='task'):
         raise ValueError(err)
 
 
-def render(template, context):
+def render(template: Union[FlexGetTemplate, str], context: Mapping, native: bool = False) -> str:
     """
     Renders a Template with `context` as its context.
 
     :param template: Template or template string to render.
     :param context: Context to render the template from.
+    :param native: If True, and the rendering result can be all native python types, not just strings.
     :return: The rendered template text.
     """
-    if isinstance(template, basestring):
+    if isinstance(template, str):
+        template_class = None
+        if native:
+            template_class = FlexGetNativeTemplate
         try:
-            template = environment.from_string(template)
+            template = environment.from_string(template, template_class=template_class)
         except TemplateSyntaxError as e:
             raise RenderError('Error in template syntax: ' + e.message)
     try:
         result = template.render(context)
     except Exception as e:
         error = RenderError('(%s) %s' % (type(e).__name__, e))
-        log.debug('Error during rendering: %s', error)
+        logger.debug('Error during rendering: {}', error)
         raise error
 
     return result
 
 
-def render_from_entry(template_string, entry):
+def render_from_entry(
+    template: Union[FlexGetTemplate, str], entry: 'Entry', native: bool = False
+) -> str:
     """Renders a Template or template string with an Entry as its context."""
 
     # Make a copy of the Entry so we can add some more fields
@@ -229,10 +275,10 @@ def render_from_entry(template_string, entry):
         # Since `task` has different meaning between entry and task scope, the `task_name` field is create to be
         # consistent
         variables['task_name'] = entry.task.name
-    return render(template_string, variables)
+    return render(template, variables, native=native)
 
 
-def render_from_task(template, task):
+def render_from_task(template: Union[FlexGetTemplate, str], task: 'Task') -> str:
     """
     Renders a Template with a task as its context.
 
@@ -244,7 +290,7 @@ def render_from_task(template, task):
     return render(template, variables)
 
 
-def evaluate_expression(expression, context):
+def evaluate_expression(expression: str, context: Mapping) -> Any:
     """
     Evaluate a jinja `expression` using a given `context` with support for `LazyDict`s (`Entry`s.)
 
