@@ -1,53 +1,56 @@
 """Contains miscellaneous helpers"""
-from __future__ import unicode_literals, division, absolute_import
-from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
-from future.moves.urllib import request
-from future.utils import PY2
-
-import logging
 import ast
 import copy
 import hashlib
 import locale
 import operator
 import os
+import queue
 import re
 import sys
-from collections import MutableMapping
-from datetime import timedelta, datetime
+from collections import defaultdict
+from collections.abc import MutableMapping
+from datetime import datetime, timedelta
+from html.entities import name2codepoint
 from pprint import pformat
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Pattern,
+    Sequence,
+    Tuple,
+    Union,
+)
+
+import requests
+from loguru import logger
 
 import flexget
-import queue
-import requests
 
-from html.entities import name2codepoint
+if TYPE_CHECKING:
+    from flexget.entry import Entry
+    from flexget.task import Task
 
-log = logging.getLogger('utils')
+logger = logger.bind(name='utils')
 
 
-def str_to_boolean(string):
+def str_to_boolean(string: str) -> bool:
     return string.lower() in ['true', '1', 't', 'y', 'yes']
 
 
-def str_to_int(string):
+def str_to_int(string: str) -> Optional[int]:
     try:
         return int(string.replace(',', ''))
     except ValueError:
         return None
 
 
-if PY2:
-    def native_str_to_text(string, **kwargs):
-        if 'encoding' not in kwargs:
-            kwargs['encoding'] = 'ascii'
-        return string.decode(**kwargs)
-else:
-    def native_str_to_text(string, **kwargs):
-        return string
-
-
-def convert_bytes(bytes):
+def convert_bytes(bytes: Union[int, float]) -> str:
     """Returns given bytes as prettified string."""
 
     bytes = float(bytes)
@@ -76,9 +79,10 @@ class MergeException(Exception):
         return repr(self.value)
 
 
-def strip_html(text):
+def strip_html(text: str) -> str:
     """Tries to strip all HTML tags from *text*. If unsuccessful returns original text."""
     from bs4 import BeautifulSoup
+
     try:
         text = ' '.join(BeautifulSoup(text).find_all(text=True))
         return ' '.join(text.split())
@@ -91,12 +95,13 @@ def strip_html(text):
 charrefpat = re.compile(r'&(#(\d+|x[\da-fA-F]+)|[\w.:-]+);?')
 
 
-def _htmldecode(text):
+def _htmldecode(text: str) -> str:
     """Decode HTML entities in the given text."""
     # From screpe.py - licensed under apache 2.0 .. should not be a problem for a MIT afaik
     if isinstance(text, str):
         uchr = chr
     else:
+
         def uchr(value):
             value > 127 and chr(value) or chr(value)
 
@@ -114,7 +119,7 @@ def _htmldecode(text):
     return charrefpat.sub(entitydecode, text)
 
 
-def decode_html(value):
+def decode_html(value: str) -> str:
     """
     :param string value: String to be html-decoded
     :returns: Html decoded string
@@ -122,35 +127,15 @@ def decode_html(value):
     return _htmldecode(value)
 
 
-def encode_html(unicode_data, encoding='ascii'):
+def encode_html(unicode_data: str, encoding: str = 'ascii') -> bytes:
     """
     Encode unicode_data for use as XML or HTML, with characters outside
     of the encoding converted to XML numeric character references.
     """
-    try:
-        return unicode_data.encode(encoding, 'xmlcharrefreplace')
-    except ValueError:
-        # ValueError is raised if there are unencodable chars in the
-        # data and the 'xmlcharrefreplace' error handler is not found.
-        # Pre-2.3 Python doesn't support the 'xmlcharrefreplace' error
-        # handler, so we'll emulate it.
-        return _xmlcharref_encode(unicode_data, encoding)
+    return unicode_data.encode(encoding, 'xmlcharrefreplace')
 
 
-def _xmlcharref_encode(unicode_data, encoding):
-    """Emulate Python 2.3's 'xmlcharrefreplace' encoding error handler."""
-    chars = []
-    # Phase through the unicode_data string one character at a time in
-    # order to catch unencodable characters:
-    for char in unicode_data:
-        try:
-            chars.append(char.encode(encoding, 'strict'))
-        except UnicodeError:
-            chars.append('&#%i;' % ord(char))
-    return ''.join(chars)
-
-
-def merge_dict_from_to(d1, d2):
+def merge_dict_from_to(d1: dict, d2: dict) -> None:
     """Merges dictionary d1 into dictionary d2. d1 will remain in original form."""
     for k, v in d1.items():
         if k in d2:
@@ -162,30 +147,23 @@ def merge_dict_from_to(d1, d2):
                 elif isinstance(v, (str, bool, int, float, type(None))):
                     pass
                 else:
-                    raise Exception('Unknown type: %s value: %s in dictionary' % (type(v), repr(v)))
-            elif (isinstance(v, (str, bool, int, float, type(None))) and
-                      isinstance(d2[k], (str, bool, int, float, type(None)))):
+                    raise Exception(
+                        'Unknown type: %s value: %s in dictionary' % (type(v), repr(v))
+                    )
+            elif isinstance(v, (str, bool, int, float, list, type(None))) and isinstance(
+                d2[k], (str, bool, int, float, list, type(None))
+            ):
                 # Allow overriding of non-container types with other non-container types
                 pass
             else:
-                raise MergeException('Merging key %s failed, conflicting datatypes %r vs. %r.' % (
-                    k, type(v).__name__, type(d2[k]).__name__))
+                raise MergeException(
+                    'Merging key %s failed, conflicting datatypes %r vs. %r.'
+                    % (k, type(v).__name__, type(d2[k]).__name__)
+                )
         else:
             d2[k] = copy.deepcopy(v)
 
 
-class SmartRedirectHandler(request.HTTPRedirectHandler):
-    def http_error_301(self, req, fp, code, msg, headers):
-        result = request.HTTPRedirectHandler.http_error_301(self, req, fp, code, msg, headers)
-        result.status = code
-        return result
-
-    def http_error_302(self, req, fp, code, msg, headers):
-        result = request.HTTPRedirectHandler.http_error_302(self, req, fp, code, msg, headers)
-        result.status = code
-        return result
-
-
 class ReList(list):
     """
     A list that stores regexps.
@@ -198,27 +176,28 @@ class ReList(list):
     # Set the default flags
     flags = re.IGNORECASE | re.UNICODE
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         """Optional :flags: keyword argument with regexp flags to compile with"""
         if 'flags' in kwargs:
             self.flags = kwargs['flags']
             del kwargs['flags']
         list.__init__(self, *args, **kwargs)
 
-    def __getitem__(self, k):
+    def __getitem__(self, k) -> Pattern:
+        # Doesn't support slices. Do we care?
         item = list.__getitem__(self, k)
         if isinstance(item, str):
             item = re.compile(item, re.IGNORECASE | re.UNICODE)
             self[k] = item
         return item
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[Pattern]:
         for i in range(len(self)):
             yield self[i]
 
 
 # Determine the encoding for io
-io_encoding = None
+io_encoding = ''
 if hasattr(sys.stdout, 'encoding'):
     io_encoding = sys.stdout.encoding
 if not io_encoding:
@@ -238,7 +217,7 @@ else:
         io_encoding = 'ascii'
 
 
-def parse_timedelta(value):
+def parse_timedelta(value: Union[timedelta, str]) -> timedelta:
     """Parse a string like '5 days' into a timedelta object. Also allows timedeltas to pass through."""
     if isinstance(value, timedelta):
         # Allow timedelta objects to pass through
@@ -257,24 +236,17 @@ def parse_timedelta(value):
         raise ValueError('Invalid time format \'%s\'' % value)
 
 
-def timedelta_total_seconds(td):
-    """replaces python 2.7+ timedelta.total_seconds()"""
-    # TODO: Remove this when we no longer support python 2.6
-    try:
-        return td.total_seconds()
-    except AttributeError:
-        return (td.days * 24 * 3600) + td.seconds + (td.microseconds / 1000000)
-
-
-def multiply_timedelta(interval, number):
+def multiply_timedelta(interval: timedelta, number: Union[int, float]) -> timedelta:
     """`timedelta`s can not normally be multiplied by floating points. This does that."""
-    return timedelta(seconds=timedelta_total_seconds(interval) * number)
+    return timedelta(seconds=interval.total_seconds() * number)
 
 
 if os.name == 'posix':
-    def pid_exists(pid):
+
+    def pid_exists(pid: int) -> bool:
         """Check whether pid exists in the current process table."""
         import errno
+
         if pid < 0:
             return False
         try:
@@ -283,10 +255,14 @@ if os.name == 'posix':
             return e.errno == errno.EPERM
         else:
             return True
+
+
 else:
-    def pid_exists(pid):
+
+    def pid_exists(pid: int) -> bool:
         import ctypes
         import ctypes.wintypes
+
         kernel32 = ctypes.windll.kernel32
         PROCESS_QUERY_INFORMATION = 0x0400
         STILL_ACTIVE = 259
@@ -305,43 +281,20 @@ else:
         # process is still running.
         return is_running or exit_code.value == STILL_ACTIVE
 
+
 _binOps = {
     ast.Add: operator.add,
     ast.Sub: operator.sub,
     ast.Mult: operator.mul,
     ast.Div: operator.truediv,
-    ast.Mod: operator.mod
+    ast.Mod: operator.mod,
 }
 
 
-def arithmeticEval(s):
-    """
-    A safe eval supporting basic arithmetic operations.
-
-    :param s: expression to evaluate
-    :return: value
-    """
-    node = ast.parse(s, mode='eval')
-
-    def _eval(node):
-        if isinstance(node, ast.Expression):
-            return _eval(node.body)
-        elif isinstance(node, ast.Str):
-            return node.s
-        elif isinstance(node, ast.Num):
-            return node.n
-        elif isinstance(node, ast.BinOp):
-            return _binOps[type(node.op)](_eval(node.left), _eval(node.right))
-        else:
-            raise Exception('Unsupported type {}'.format(node))
-
-    return _eval(node.body)
-
-
 class TimedDict(MutableMapping):
     """Acts like a normal dict, but keys will only remain in the dictionary for a specified time span."""
 
-    def __init__(self, cache_time='5 minutes'):
+    def __init__(self, cache_time: Union[timedelta, str] = '5 minutes'):
         self.cache_time = parse_timedelta(cache_time)
         self._store = dict()
         self._last_prune = datetime.now()
@@ -379,11 +332,14 @@ class TimedDict(MutableMapping):
 
     def __repr__(self):
         return '%s(%r)' % (
-            self.__class__.__name__, dict(list(zip(self._store, (v[1] for v in list(self._store.values()))))))
+            self.__class__.__name__,
+            dict(list(zip(self._store, (v[1] for v in list(self._store.values()))))),
+        )
 
 
 class BufferQueue(queue.Queue):
     """Used in place of a file-like object to capture text and access it safely from another thread."""
+
     # Allow access to the Empty error from here
     Empty = queue.Empty
 
@@ -391,24 +347,14 @@ class BufferQueue(queue.Queue):
         self.put(line)
 
 
-def singleton(cls):
-    instances = {}
-
-    def getinstance(*args, **kwargs):
-        if cls not in instances:
-            instances[cls] = cls(*args, **kwargs)
-        return instances[cls]
-
-    return getinstance
-
-
-def split_title_year(title):
+def split_title_year(title: str) -> Tuple[str, Optional[int]]:
     """Splits title containing a year into a title, year pair."""
     if not title:
-        return
+        return '', None
     if not re.search(r'\d{4}', title):
         return title, None
-    match = re.search(r'(.*?)\(?(\d{4})?\)?$', title)
+    # We only recognize years from the 2nd and 3rd millennium, FlexGetters from the year 3000 be damned!
+    match = re.search(r'(.*?)\(?([12]\d{3})?\)?$', title)
 
     title = match.group(1).strip()
     year_match = match.group(2)
@@ -424,7 +370,7 @@ def split_title_year(title):
     return title, year
 
 
-def get_latest_flexget_version_number():
+def get_latest_flexget_version_number() -> Optional[str]:
     """
     Return latest Flexget version from https://pypi.python.org/pypi/FlexGet/json
     """
@@ -435,11 +381,11 @@ def get_latest_flexget_version_number():
         return
 
 
-def get_current_flexget_version():
+def get_current_flexget_version() -> str:
     return flexget.__version__
 
 
-def parse_filesize(text_size, si=True):
+def parse_filesize(text_size: str, si: bool = True) -> float:
     """
     Parses a data size and returns its value in mebibytes
 
@@ -451,7 +397,9 @@ def parse_filesize(text_size, si=True):
     """
     prefix_order = {'': 0, 'k': 1, 'm': 2, 'g': 3, 't': 4, 'p': 5}
 
-    parsed_size = re.match('(\d+(?:[.,\s]\d+)*)(?:\s*)((?:[ptgmk]i?)?b)', text_size.strip().lower(), flags=re.UNICODE)
+    parsed_size = re.match(
+        r'(\d+(?:[.,\s]\d+)*)(?:\s*)((?:[ptgmk]i?)?b)', text_size.strip().lower(), flags=re.UNICODE
+    )
     if not parsed_size:
         raise ValueError('%s does not look like a file size' % text_size)
     amount = parsed_size.group(1)
@@ -470,7 +418,7 @@ def parse_filesize(text_size, si=True):
     return (amount * (base ** order)) / 1024 ** 2
 
 
-def get_config_hash(config):
+def get_config_hash(config: Any) -> str:
     """
     :param dict config: Configuration
     :return: MD5 hash for *config*
@@ -482,7 +430,7 @@ def get_config_hash(config):
         return hashlib.md5(str(config).encode('utf-8')).hexdigest()
 
 
-def get_config_as_array(config, key):
+def get_config_as_array(config: dict, key: str) -> list:
     """
     Return configuration key as array, even if given as a single string
     :param dict config: Configuration
@@ -490,12 +438,14 @@ def get_config_as_array(config, key):
     :return: Array
     """
     v = config.get(key, [])
-    if isinstance(v, str):
+    if not isinstance(v, list):
         return [v]
     return v
 
 
-def parse_episode_identifier(ep_id, identify_season=False):
+def parse_episode_identifier(
+    ep_id: Union[str, int], identify_season: bool = False
+) -> Tuple[str, str]:
     """
     Parses series episode identifier, raises ValueError if it fails
 
@@ -528,10 +478,28 @@ def parse_episode_identifier(ep_id, identify_season=False):
             error = '`%s` is not a valid episode identifier.' % ep_id
     if error:
         raise ValueError(error)
-    return (identified_by, entity_type)
+    return identified_by, entity_type
 
 
-def aggregate_inputs(task, inputs):
+def group_entries(entries: Iterable['Entry'], identifier: str) -> Dict[str, List['Entry']]:
+    from flexget.utils.template import RenderError
+
+    grouped_entries = defaultdict(list)
+
+    # Group by Identifier
+    for entry in entries:
+        try:
+            rendered_id = entry.render(identifier)
+        except RenderError:
+            continue
+        if not rendered_id:
+            continue
+        grouped_entries[rendered_id.lower().strip()].append(entry)
+
+    return grouped_entries
+
+
+def aggregate_inputs(task: 'Task', inputs: List[dict]) -> List['Entry']:
     from flexget import plugin
 
     entries = []
@@ -541,32 +509,34 @@ def aggregate_inputs(task, inputs):
     for item in inputs:
         for input_name, input_config in item.items():
             input = plugin.get_plugin_by_name(input_name)
-            if input.api_ver == 1:
-                raise plugin.PluginError('Plugin %s does not support API v2' % input_name)
             method = input.phase_handlers['input']
             try:
                 result = method(task, input_config)
             except plugin.PluginError as e:
-                log.warning('Error during input plugin %s: %s', input_name, e)
+                logger.warning('Error during input plugin {}: {}', input_name, e)
                 continue
 
             if not result:
-                log.warning('Input %s did not return anything', input_name)
+                logger.warning('Input {} did not return anything', input_name)
                 continue
 
             for entry in result:
                 urls = ([entry['url']] if entry.get('url') else []) + entry.get('urls', [])
 
                 if any(url in entry_urls for url in urls):
-                    log.debug('URL for `%s` already in entry list, skipping.', entry['title'])
+                    logger.debug('URL for `{}` already in entry list, skipping.', entry['title'])
                     continue
 
                 if entry['title'] in entry_titles:
-                    log.debug('Ignored duplicate title `%s`', entry['title'])  # TODO: should combine?
+                    logger.debug(
+                        'Ignored duplicate title `{}`', entry['title']
+                    )  # TODO: should combine?
                     continue
 
                 if entry.get('location') and entry['location'] in entry_locations:
-                    log.debug('Ignored duplicate location `%s`', entry['location'])  # TODO: should combine?
+                    logger.debug(
+                        'Ignored duplicate location `{}`', entry['location']
+                    )  # TODO: should combine?
                     continue
 
                 entries.append(entry)
@@ -576,3 +546,11 @@ def aggregate_inputs(task, inputs):
                     entry_locations.add(entry['location'])
 
     return entries
+
+
+# Mainly used due to Too Many Variables error if we use too many variables at a time in the in_ clause.
+# SQLite supports up to 999 by default. Ubuntu, Arch and macOS set this limit to 250,000 though, so it's a rare issue.
+def chunked(seq: Sequence, limit: int = 900) -> Iterator[Sequence]:
+    """Helper to divide our expired lists into sizes sqlite can handle in a query. (<1000)"""
+    for i in range(0, len(seq), limit):
+        yield seq[i : i + limit]
